@@ -228,7 +228,7 @@ static int add_files_in_sfile(int vid){
   }else{
     xCmp = fossil_stricmp;
   }
-  db_prepare(&loop, "SELECT x FROM sfile ORDER BY x");
+  db_prepare(&loop, "SELECT pathname FROM sfile ORDER BY pathname");
   while( db_step(&loop)==SQLITE_ROW ){
     const char *zToAdd = db_column_text(&loop, 0);
     if( fossil_strcmp(zToAdd, zRepo)==0 ) continue;
@@ -278,6 +278,7 @@ static int add_files_in_sfile(int vid){
 **                            comma separated list of glob patterns.
 **    --clean <CSG>           Also ignore files matching patterns from
 **                            the comma separated list of glob patterns.
+**    --no-dir-symlinks       Disables support for directory symlinks.
 **
 ** See also: addremove, rm
 */
@@ -309,7 +310,7 @@ void add_cmd(void){
   if( db_get_boolean("dotfiles", 0) ) scanFlags |= SCAN_ALL;
   vid = db_lget_int("checkout",0);
   db_begin_transaction();
-  db_multi_exec("CREATE TEMP TABLE sfile(x TEXT PRIMARY KEY %s)",
+  db_multi_exec("CREATE TEMP TABLE sfile(pathname TEXT PRIMARY KEY %s)",
                 filename_collation());
   pClean = glob_create(zCleanFlag);
   pIgnore = glob_create(zIgnoreFlag);
@@ -351,7 +352,7 @@ void add_cmd(void){
         }
       }
       db_multi_exec(
-         "INSERT OR IGNORE INTO sfile(x) VALUES(%Q)",
+         "INSERT OR IGNORE INTO sfile(pathname) VALUES(%Q)",
          zTreeName
       );
     }
@@ -443,6 +444,7 @@ static void process_files_to_remove(
 **   --hard                  Remove files from the checkout.
 **   --case-sensitive <BOOL> Override the case-sensitive setting.
 **   -n|--dry-run            If given, display instead of run actions.
+**   --no-dir-symlinks       Disables support for directory symlinks.
 **
 ** See also: addremove, add
 */
@@ -476,7 +478,7 @@ void delete_cmd(void){
     removeFiles = FOSSIL_MV_RM_FILE;
 #endif
   }
-  db_multi_exec("CREATE TEMP TABLE sfile(x TEXT PRIMARY KEY %s)",
+  db_multi_exec("CREATE TEMP TABLE sfile(pathname TEXT PRIMARY KEY %s)",
                 filename_collation());
   for(i=2; i<g.argc; i++){
     Blob treeName;
@@ -496,7 +498,7 @@ void delete_cmd(void){
     blob_reset(&treeName);
   }
 
-  db_prepare(&loop, "SELECT x FROM sfile");
+  db_prepare(&loop, "SELECT pathname FROM sfile");
   while( db_step(&loop)==SQLITE_ROW ){
     fossil_print("DELETED %s\n", db_column_text(&loop, 0));
     if( removeFiles ) add_file_to_remove(db_column_text(&loop, 0));
@@ -624,6 +626,7 @@ const char *filename_collation(void){
 **   --clean <CSG>           Also ignore files matching patterns from
 **                           the comma separated list of glob patterns.
 **   -n|--dry-run            If given, display instead of run actions.
+**   --no-dir-symlinks       Disables support for directory symlinks.
 **
 ** See also: add, rm
 */
@@ -672,7 +675,7 @@ void addremove_cmd(void){
   ** --ignore or ignore-glob patterns and dot-files.  Then add all of
   ** the files in the sfile temp table to the set of managed files.
   */
-  db_multi_exec("CREATE TEMP TABLE sfile(x TEXT PRIMARY KEY %s)",
+  db_multi_exec("CREATE TEMP TABLE sfile(pathname TEXT PRIMARY KEY %s)",
                 filename_collation());
   n = strlen(g.zLocalRoot);
   blob_init(&path, g.zLocalRoot, n-1);
@@ -851,6 +854,7 @@ static void process_files_to_move(
 **   --hard                  Move files within the checkout.
 **   --case-sensitive <BOOL> Override the case-sensitive setting.
 **   -n|--dry-run            If given, display instead of run actions.
+**   --no-dir-symlinks       Disables support for directory symlinks.
 **
 ** See also: changes, status
 */
