@@ -23,6 +23,10 @@
 */
 #include "config.h"
 #ifdef _WIN32
+# if !defined(_WIN32_WINNT)
+#  define _WIN32_WINNT 0x0501
+# endif
+# include <winsock2.h>
 # include <ws2tcpip.h>
 #else
 # include <sys/socket.h>
@@ -41,6 +45,7 @@
 #include <stdlib.h>
 #include <unistd.h>
 #include "cgi.h"
+#include "cygsup.h"
 
 #if INTERFACE
 /*
@@ -52,6 +57,7 @@
 #define PD(x,y)     cgi_parameter((x),(y))
 #define PT(x)       cgi_parameter_trimmed((x),0)
 #define PDT(x,y)    cgi_parameter_trimmed((x),(y))
+#define PB(x)       cgi_parameter_boolean(x)
 
 
 /*
@@ -60,17 +66,24 @@
 #define CGI_HEADER   0
 #define CGI_BODY     1
 
+/*
+** Flags for SSH HTTP clients
+*/
+#define CGI_SSH_CLIENT           0x0001     /* Client is SSH */
+#define CGI_SSH_COMPAT           0x0002     /* Compat for old SSH transport */
+#define CGI_SSH_FOSSIL           0x0004     /* Use new Fossil SSH transport */
+
 #endif /* INTERFACE */
 
 /*
 ** The HTTP reply is generated in two pieces: the header and the body.
-** These pieces are generated separately because they are not necessary
+** These pieces are generated separately because they are not necessarily
 ** produced in order.  Parts of the header might be built after all or
 ** part of the body.  The header and body are accumulated in separate
 ** Blob structures then output sequentially once everything has been
 ** built.
 **
-** The cgi_destination() interface switch between the buffers.
+** The cgi_destination() interface switches between the buffers.
 */
 static Blob cgiContent[2] = { BLOB_INITIALIZER, BLOB_INITIALIZER };
 static Blob *pContent = &cgiContent[0];
@@ -101,6 +114,9 @@ void cgi_destination(int dest){
 int cgi_header_contains(const char *zNeedle){
   return strstr(blob_str(&cgiContent[0]), zNeedle)!=0;
 }
+int cgi_body_contains(const char *zNeedle){
+  return strstr(blob_str(&cgiContent[1]), zNeedle)!=0;
+}
 
 /*
 ** Append reply content to what already exists.
@@ -259,11 +275,21 @@ static int check_cache_control(void){
       if(zTok) return 1;
     }
   }
-  
+
   return 0;
 }
 #endif
 
+/*
+** Return true if the response should be sent with Content-Encoding: gzip.
+*/
+static int is_gzippable(void){
+  if( strstr(PD("HTTP_ACCEPT_ENCODING", ""), "gzip")==0 ) return 0;
+  return strncmp(zContentType, "text/", 5)==0
+    || sqlite3_strglob("application/*xml", zContentType)==0
+    || sqlite3_strglob("application/*javascript", zContentType)==0;
+}
+
 /*
 ** Do a normal HTTP reply
 */
@@ -289,6 +315,7 @@ void cgi_reply(void){
     fprintf(g.httpOut, "HTTP/1.0 %d %s\r\n", iReplyStatus, zReplyStatus);
     fprintf(g.httpOut, "Date: %s\r\n", cgi_rfc822_datestamp(time(0)));
     fprintf(g.httpOut, "Connection: close\r\n");
+    fprintf(g.httpOut, "X-UA-Compatible: IE=edge\r\n");
   }else{
     fprintf(g.httpOut, "Status: %d %s\r\n", iReplyStatus, zReplyStatus);
   }
@@ -324,9 +351,7 @@ void cgi_reply(void){
     ** stale cache is the least of the problem. So we provide an Expires
     ** header set to a reasonable period (default: one week).
     */
-    /*time_t expires = time(0) + atoi(db_config("constant_expires","604800"));*/
-    time_t expires = time(0) + 604800;
-    fprintf(g.httpOut, "Expires: %s\r\n", cgi_rfc822_datestamp(expires));
+    fprintf(g.httpOut, "Cache-control: max-age=28800\r\n");
   }else{
     fprintf(g.httpOut, "Cache-control: no-cache\r\n");
   }
@@ -341,6 +366,18 @@ void cgi_reply(void){
   }
 
   if( iReplyStatus != 304 ) {
+    if( is_gzippable() ){
+      int i;
+      gzip_begin(0);
+      for( i=0; i<2; i++ ){
+        int size = blob_size(&cgiContent[i]);
+        if( size>0 ) gzip_step(blob_buffer(&cgiContent[i]), size);
+        blob_reset(&cgiContent[i]);
+      }
+      gzip_finish(&cgiContent[0]);
+      fprintf(g.httpOut, "Content-Encoding: gzip\r\n");
+      fprintf(g.httpOut, "Vary: Accept-Encoding\r\n");
+    }
     total_size = blob_size(&cgiContent[0]) + blob_size(&cgiContent[1]);
     fprintf(g.httpOut, "Content-Length: %d\r\n", total_size);
   }else{
@@ -365,7 +402,11 @@ void cgi_reply(void){
 **
 ** The URL must be relative to the base of the fossil server.
 */
-NORETURN void cgi_redirect(const char *zURL){
+NORETURN static void cgi_redirect_with_status(
+  const char *zURL,
+  int iStat,
+  const char *zStat
+){
   char *zLocation;
   CGIDEBUG(("redirect to %s\n", zURL));
   if( strncmp(zURL,"http:",5)==0 || strncmp(zURL,"https:",6)==0 ){
@@ -381,11 +422,17 @@ NORETURN void cgi_redirect(const char *zURL){
   cgi_append_header(zLocation);
   cgi_reset_content();
   cgi_printf("<html>\n<p>Redirect to %h</p>\n</html>\n", zLocation);
-  cgi_set_status(302, "Moved Temporarily");
+  cgi_set_status(iStat, zStat);
   free(zLocation);
   cgi_reply();
   fossil_exit(0);
 }
+NORETURN void cgi_redirect(const char *zURL){
+  cgi_redirect_with_status(zURL, 302, "Moved Temporarily");
+}
+NORETURN void cgi_redirect_with_method(const char *zURL){
+  cgi_redirect_with_status(zURL, 307, "Temporary Redirect");
+}
 NORETURN void cgi_redirectf(const char *zFormat, ...){
   va_list ap;
   va_start(ap, zFormat);
@@ -405,6 +452,8 @@ static struct QParam {   /* One entry for each query parameter or cookie */
   const char *zName;        /* Parameter or cookie name */
   const char *zValue;       /* Value of the query parameter or cookie */
   int seq;                  /* Order of insertion */
+  char isQP;                /* True for query parameters */
+  char cTag;                /* Tag on query parameters */
 } *aParamQP;             /* An array of all parameters and cookies */
 
 /*
@@ -415,7 +464,7 @@ static struct QParam {   /* One entry for each query parameter or cookie */
 ** zName and zValue are not copied and must not change or be
 ** deallocated after this routine returns.
 */
-void cgi_set_parameter_nocopy(const char *zName, const char *zValue){
+void cgi_set_parameter_nocopy(const char *zName, const char *zValue, int isQP){
   if( nAllocQP<=nUsedQP ){
     nAllocQP = nAllocQP*2 + 10;
     if( nAllocQP>1000 ){
@@ -430,6 +479,8 @@ void cgi_set_parameter_nocopy(const char *zName, const char *zValue){
     fprintf(stderr, "# cgi: %s = [%s]\n", zName, zValue);
   }
   aParamQP[nUsedQP].seq = seqQP++;
+  aParamQP[nUsedQP].isQP = isQP;
+  aParamQP[nUsedQP].cTag = 0;
   nUsedQP++;
   sortQP = 1;
 }
@@ -442,7 +493,10 @@ void cgi_set_parameter_nocopy(const char *zName, const char *zValue){
 ** Copies are made of both the zName and zValue parameters.
 */
 void cgi_set_parameter(const char *zName, const char *zValue){
-  cgi_set_parameter_nocopy(mprintf("%s",zName), mprintf("%s",zValue));
+  cgi_set_parameter_nocopy(mprintf("%s",zName), mprintf("%s",zValue), 0);
+}
+void cgi_set_query_parameter(const char *zName, const char *zValue){
+  cgi_set_parameter_nocopy(mprintf("%s",zName), mprintf("%s",zValue), 1);
 }
 
 /*
@@ -456,7 +510,47 @@ void cgi_replace_parameter(const char *zName, const char *zValue){
       return;
     }
   }
-  cgi_set_parameter_nocopy(zName, zValue);
+  cgi_set_parameter_nocopy(zName, zValue, 0);
+}
+void cgi_replace_query_parameter(const char *zName, const char *zValue){
+  int i;
+  for(i=0; i<nUsedQP; i++){
+    if( fossil_strcmp(aParamQP[i].zName,zName)==0 ){
+      aParamQP[i].zValue = zValue;
+      assert( aParamQP[i].isQP );
+      return;
+    }
+  }
+  cgi_set_parameter_nocopy(zName, zValue, 1);
+}
+
+/*
+** Delete a parameter.
+*/
+void cgi_delete_parameter(const char *zName){
+  int i;
+  for(i=0; i<nUsedQP; i++){
+    if( fossil_strcmp(aParamQP[i].zName,zName)==0 ){
+      --nUsedQP;
+      if( i<nUsedQP ){
+        memmove(aParamQP+i, aParamQP+i+1, sizeof(*aParamQP)*(nUsedQP-i));
+      }
+      return;
+    }
+  }
+}
+void cgi_delete_query_parameter(const char *zName){
+  int i;
+  for(i=0; i<nUsedQP; i++){
+    if( fossil_strcmp(aParamQP[i].zName,zName)==0 ){
+      assert( aParamQP[i].isQP );
+      --nUsedQP;
+      if( i<nUsedQP ){
+        memmove(aParamQP+i, aParamQP+i+1, sizeof(*aParamQP)*(nUsedQP-i));
+      }
+      return;
+    }
+  }
 }
 
 /*
@@ -464,9 +558,9 @@ void cgi_replace_parameter(const char *zName, const char *zValue){
 ** must be made of zValue.
 */
 void cgi_setenv(const char *zName, const char *zValue){
-  cgi_set_parameter_nocopy(zName, mprintf("%s",zValue));
+  cgi_set_parameter_nocopy(zName, mprintf("%s",zValue), 0);
 }
- 
+
 
 /*
 ** Add a list of query parameters or cookies to the parameter set.
@@ -494,6 +588,7 @@ void cgi_setenv(const char *zName, const char *zValue){
 ** returns or it will corrupt the parameter table.
 */
 static void add_param_list(char *z, int terminator){
+  int isQP = terminator=='&';
   while( *z ){
     char *zName;
     char *zValue;
@@ -515,7 +610,7 @@ static void add_param_list(char *z, int terminator){
       zValue = "";
     }
     if( fossil_islower(zName[0]) ){
-      cgi_set_parameter_nocopy(zName, zValue);
+      cgi_set_parameter_nocopy(zName, zValue, isQP);
     }
 #ifdef FOSSIL_ENABLE_JSON
     json_setenv( zName, cson_value_new_string(zValue,strlen(zValue)) );
@@ -583,7 +678,7 @@ static char *get_bounded_content(
   }
   *pz = &z[i];
   get_line_from_string(pz, pLen);
-  return z;      
+  return z;
 }
 
 /*
@@ -659,16 +754,16 @@ static void process_multipart_form_data(char *z, int len){
       int nContent = 0;
       zValue = get_bounded_content(&z, &len, zBoundry, &nContent);
       if( zName && zValue && fossil_islower(zName[0]) ){
-        cgi_set_parameter_nocopy(zName, zValue);
+        cgi_set_parameter_nocopy(zName, zValue, 1);
         if( showBytes ){
           cgi_set_parameter_nocopy(mprintf("%s:bytes", zName),
-               mprintf("%d",nContent));
+               mprintf("%d",nContent), 1);
         }
       }
       zName = 0;
       showBytes = 0;
     }else{
-      nArg = tokenize_line(zLine, sizeof(azArg)/sizeof(azArg[0]), azArg);
+      nArg = tokenize_line(zLine, count(azArg), azArg);
       for(i=0; i<nArg; i++){
         int c = fossil_tolower(azArg[i][0]);
         int n = strlen(azArg[i]);
@@ -679,18 +774,18 @@ static void process_multipart_form_data(char *z, int len){
         }else if( c=='f' && sqlite3_strnicmp(azArg[i],"filename=",n)==0 ){
           char *z = azArg[++i];
           if( zName && z && fossil_islower(zName[0]) ){
-            cgi_set_parameter_nocopy(mprintf("%s:filename",zName), z);
+            cgi_set_parameter_nocopy(mprintf("%s:filename",zName), z, 1);
           }
           showBytes = 1;
         }else if( c=='c' && sqlite3_strnicmp(azArg[i],"content-type:",n)==0 ){
           char *z = azArg[++i];
           if( zName && z && fossil_islower(zName[0]) ){
-            cgi_set_parameter_nocopy(mprintf("%s:mimetype",zName), z);
+            cgi_set_parameter_nocopy(mprintf("%s:mimetype",zName), z, 1);
           }
         }
       }
     }
-  }        
+  }
 }
 
 
@@ -718,7 +813,7 @@ static int cson_data_source_FILE_n( void * state,
       if( st->pos >= st->len ){
         *n = 0;
         return 0;
-      } else if( !*n || ((st->pos + *n) > st->len) ){
+      }else if( !*n || ((st->pos + *n) > st->len) ){
         return cson_rc.RangeError;
       }else{
         unsigned int rsz = (unsigned int)fread( dest, 1, *n, st->fh );
@@ -748,6 +843,7 @@ void cgi_parse_POST_JSON( FILE * zIn, unsigned int contentLen ){
   CgiPostReadState state;
   cson_parse_opt popt = cson_parse_opt_empty;
   cson_parse_info pinfo = cson_parse_info_empty;
+  assert(g.json.gc.a && "json_main_bootstrap() was not called!");
   popt.maxDepth = 15;
   state.fh = zIn;
   state.len = contentLen;
@@ -785,28 +881,93 @@ void cgi_parse_POST_JSON( FILE * zIn, unsigned int contentLen ){
 }
 #endif /* FOSSIL_ENABLE_JSON */
 
+/*
+** Log HTTP traffic to a file.  Begin the log on first use.  Close the log
+** when the argument is NULL.
+*/
+void cgi_trace(const char *z){
+  static FILE *pLog = 0;
+  if( g.fHttpTrace==0 ) return;
+  if( z==0 ){
+    if( pLog ) fclose(pLog);
+    pLog = 0;
+    return;
+  }
+  if( pLog==0 ){
+    char zFile[50];
+    unsigned r;
+    sqlite3_randomness(sizeof(r), &r);
+    sqlite3_snprintf(sizeof(zFile), zFile, "httplog-%08x.txt", r);
+    pLog = fossil_fopen(zFile, "wb");
+    if( pLog ){
+      fprintf(stderr, "# open log on %s\n", zFile);
+    }else{
+      fprintf(stderr, "# failed to open %s\n", zFile);
+      return;
+    }
+  }
+  fputs(z, pLog);
+}
+
+/* Forward declaration */
+static NORETURN void malformed_request(const char *zMsg);
 
 /*
 ** Initialize the query parameter database.  Information is pulled from
 ** the QUERY_STRING environment variable (if it exists), from standard
 ** input if there is POST data, and from HTTP_COOKIE.
+**
+** REQUEST_URI, PATH_INFO, and SCRIPT_NAME are related as follows:
+**
+**      REQUEST_URI == SCRIPT_NAME + PATH_INFO
+**
+** Where "+" means concatenate.  Fossil requires SCRIPT_NAME.  If
+** REQUEST_URI is provided but PATH_INFO is not, then PATH_INFO is
+** computed from REQUEST_URI and SCRIPT_NAME.  If PATH_INFO is provided
+** but REQUEST_URI is not, then compute REQUEST_URI from PATH_INFO and
+** SCRIPT_NAME.  If neither REQUEST_URI nor PATH_INFO are provided, then
+** assume that PATH_INFO is an empty string and set REQUEST_URI equal
+** to PATH_INFO.
+**
+** SCGI typically omits PATH_INFO.  CGI sometimes omits REQUEST_URI and
+** PATH_INFO when it is empty.
 */
 void cgi_init(void){
   char *z;
   const char *zType;
   int len;
+  const char *zRequestUri = cgi_parameter("REQUEST_URI",0);
+  const char *zScriptName = cgi_parameter("SCRIPT_NAME",0);
+  const char *zPathInfo = cgi_parameter("PATH_INFO",0);
+
 #ifdef FOSSIL_ENABLE_JSON
   json_main_bootstrap();
 #endif
   g.isHTTP = 1;
   cgi_destination(CGI_BODY);
+  if( zScriptName==0 ) malformed_request("missing SCRIPT_NAME");
+  if( zRequestUri==0 ){
+    const char *z = zPathInfo;
+    if( zPathInfo==0 ){
+      malformed_request("missing PATH_INFO and/or REQUEST_URI");
+    }
+    if( z[0]=='/' ) z++;
+    zRequestUri = mprintf("%s/%s", zScriptName, z);
+    cgi_set_parameter("REQUEST_URI", zRequestUri);
+  }
+  if( zPathInfo==0 ){
+    int i, j;
+    for(i=0; zRequestUri[i]==zScriptName[i] && zRequestUri[i]; i++){}
+    for(j=i; zRequestUri[j] && zRequestUri[j]!='?'; j++){}
+    cgi_set_parameter("PATH_INFO", mprintf("%.*s", j-i, zRequestUri+i));
+  }
 
   z = (char*)P("HTTP_COOKIE");
   if( z ){
     z = mprintf("%s",z);
     add_param_list(z, ';');
   }
-  
+
   z = (char*)P("QUERY_STRING");
   if( z ){
     z = mprintf("%s",z);
@@ -820,13 +981,14 @@ void cgi_init(void){
 
   len = atoi(PD("CONTENT_LENGTH", "0"));
   g.zContentType = zType = P("CONTENT_TYPE");
+  blob_zero(&g.cgiIn);
   if( len>0 && zType ){
-    blob_zero(&g.cgiIn);
-    if( fossil_strcmp(zType,"application/x-www-form-urlencoded")==0 
+    if( fossil_strcmp(zType,"application/x-www-form-urlencoded")==0
          || strncmp(zType,"multipart/form-data",19)==0 ){
       z = fossil_malloc( len+1 );
       len = fread(z, 1, len, g.httpIn);
       z[len] = 0;
+      cgi_trace(z);
       if( zType[0]=='a' ){
         add_param_list(z, '&');
       }else{
@@ -851,7 +1013,7 @@ void cgi_init(void){
       - See if fossil really needs g.cgiIn to be set for this purpose
       (i don't think it does). If it does then fill g.cgiIn and
       refactor to parse the JSON from there.
-      
+
       - After parsing POST JSON, copy the "first layer" of keys/values
       to cgi_setenv(), honoring the upper-case distinction used
       in add_param_list(). However...
@@ -936,10 +1098,10 @@ const char *cgi_parameter(const char *zName, const char *zDefault){
   ** letter, then check to see if there is an environment variable
   ** with the given name.
   */
-  if( fossil_isupper(zName[0]) ){
+  if( zName && fossil_isupper(zName[0]) ){
     const char *zValue = fossil_getenv(zName);
     if( zValue ){
-      cgi_set_parameter_nocopy(zName, zValue);
+      cgi_set_parameter_nocopy(zName, zValue, 0);
       CGIDEBUG(("env-match [%s] = [%s]\n", zName, zValue));
       return zValue;
     }
@@ -965,9 +1127,19 @@ char *cgi_parameter_trimmed(const char *zName, const char *zDefault){
   return zOut;
 }
 
+/*
+** Return true if the CGI parameter zName exists and is not equal to 0,
+** or "no" or "off".
+*/
+int cgi_parameter_boolean(const char *zName){
+  const char *zIn = cgi_parameter(zName, 0);
+  if( zIn==0 ) return 0;
+  return zIn[0]==0 || is_truth(zIn);
+}
+
 /*
 ** Return the name of the i-th CGI parameter.  Return NULL if there
-** are fewer than i registered CGI parmaeters.
+** are fewer than i registered CGI parameters.
 */
 const char *cgi_parameter_name(int i){
   if( i>=0 && i<nUsedQP ){
@@ -1041,6 +1213,49 @@ void cgi_print_all(int showAll){
   }
 }
 
+/*
+** Export all untagged query parameters (but not cookies or environment
+** variables) as hidden values of a form.
+*/
+void cgi_query_parameters_to_hidden(void){
+  int i;
+  const char *zN, *zV;
+  for(i=0; i<nUsedQP; i++){
+    if( aParamQP[i].isQP==0 || aParamQP[i].cTag ) continue;
+    zN = aParamQP[i].zName;
+    zV = aParamQP[i].zValue;
+    @ <input type="hidden" name="%h(zN)" value="%h(zV)">
+  }
+}
+
+/*
+** Export all untagged query parameters (but not cookies or environment
+** variables) to the HQuery object.
+*/
+void cgi_query_parameters_to_url(HQuery *p){
+  int i;
+  for(i=0; i<nUsedQP; i++){
+    if( aParamQP[i].isQP==0 || aParamQP[i].cTag ) continue;
+    url_add_parameter(p, aParamQP[i].zName, aParamQP[i].zValue);
+  }
+}
+
+/*
+** Tag query parameter zName so that it is not exported by
+** cgi_query_parameters_to_hidden().  Or if zName==0, then
+** untag all query parameters.
+*/
+void cgi_tag_query_parameter(const char *zName){
+  int i;
+  if( zName==0 ){
+    for(i=0; i<nUsedQP; i++) aParamQP[i].cTag = 0;
+  }else{
+    for(i=0; i<nUsedQP; i++){
+      if( strcmp(zName,aParamQP[i].zName)==0 ) aParamQP[i].cTag = 1;
+    }
+  }
+}
+
 /*
 ** This routine works like "printf" except that it has the
 ** extra formatting capabilities such as %h and %t.
@@ -1064,10 +1279,10 @@ void cgi_vprintf(const char *zFormat, va_list ap){
 /*
 ** Send a reply indicating that the HTTP request was malformed
 */
-static NORETURN void malformed_request(void){
+static NORETURN void malformed_request(const char *zMsg){
   cgi_set_status(501, "Not Implemented");
   cgi_printf(
-    "<html><body>Unrecognized HTTP Request</body></html>\n"
+    "<html><body><p>Bad Request: %s</p></body></html>\n", zMsg
   );
   cgi_reply();
   fossil_exit(0);
@@ -1104,6 +1319,20 @@ NORETURN void cgi_panic(const char *zFormat, ...){
   }
 }
 
+/* z[] is the value of an X-FORWARDED-FOR: line in an HTTP header.
+** Return a pointer to a string containing the real IP address, or a
+** NULL pointer to stick with the IP address previously computed and
+** loaded into g.zIpAddr.
+*/
+static const char *cgi_accept_forwarded_for(const char *z){
+  int i;
+  if( fossil_strcmp(g.zIpAddr, "127.0.0.1")!=0 ) return 0;
+
+  i = strlen(z)-1;
+  while( i>=0 && z[i]!=',' && !fossil_isspace(z[i]) ) i--;
+  return &z[++i];
+}
+
 /*
 ** Remove the first space-delimited token from a string and return
 ** a pointer to it.  Add a NULL to the string to terminate the token.
@@ -1145,44 +1374,49 @@ void cgi_handle_http_request(const char *zIpAddr){
   char zLine[2000];     /* A single line of input. */
   g.fullHttpReply = 1;
   if( fgets(zLine, sizeof(zLine),g.httpIn)==0 ){
-    malformed_request();
+    malformed_request("missing HTTP header");
   }
+  blob_append(&g.httpHeader, zLine, -1);
+  cgi_trace(zLine);
   zToken = extract_token(zLine, &z);
   if( zToken==0 ){
-    malformed_request();
+    malformed_request("malformed HTTP header");
   }
   if( fossil_strcmp(zToken,"GET")!=0 && fossil_strcmp(zToken,"POST")!=0
       && fossil_strcmp(zToken,"HEAD")!=0 ){
-    malformed_request();
+    malformed_request("unsupported HTTP method");
   }
   cgi_setenv("GATEWAY_INTERFACE","CGI/1.0");
   cgi_setenv("REQUEST_METHOD",zToken);
   zToken = extract_token(z, &z);
   if( zToken==0 ){
-    malformed_request();
+    malformed_request("malformed URL in HTTP header");
   }
   cgi_setenv("REQUEST_URI", zToken);
+  cgi_setenv("SCRIPT_NAME", "");
   for(i=0; zToken[i] && zToken[i]!='?'; i++){}
   if( zToken[i] ) zToken[i++] = 0;
   cgi_setenv("PATH_INFO", zToken);
   cgi_setenv("QUERY_STRING", &zToken[i]);
   if( zIpAddr==0 &&
-        getpeername(fileno(g.httpIn), (struct sockaddr*)&remoteName, 
+        getpeername(fileno(g.httpIn), (struct sockaddr*)&remoteName,
                                 &size)>=0
   ){
     zIpAddr = inet_ntoa(remoteName.sin_addr);
   }
-  if( zIpAddr ){   
+  if( zIpAddr ){
     cgi_setenv("REMOTE_ADDR", zIpAddr);
     g.zIpAddr = mprintf("%s", zIpAddr);
   }
- 
+
   /* Get all the optional fields that follow the first line.
   */
   while( fgets(zLine,sizeof(zLine),g.httpIn) ){
     char *zFieldName;
     char *zVal;
 
+    cgi_trace(zLine);
+    blob_append(&g.httpHeader, zLine, -1);
     zFieldName = extract_token(zLine,&zVal);
     if( zFieldName==0 || *zFieldName==0 ) break;
     while( fossil_isspace(*zVal) ){ zVal++; }
@@ -1192,7 +1426,9 @@ void cgi_handle_http_request(const char *zIpAddr){
     for(i=0; zFieldName[i]; i++){
       zFieldName[i] = fossil_tolower(zFieldName[i]);
     }
-    if( fossil_strcmp(zFieldName,"content-length:")==0 ){
+    if( fossil_strcmp(zFieldName,"accept-encoding:")==0 ){
+      cgi_setenv("HTTP_ACCEPT_ENCODING", zVal);
+    }else if( fossil_strcmp(zFieldName,"content-length:")==0 ){
       cgi_setenv("CONTENT_LENGTH", zVal);
     }else if( fossil_strcmp(zFieldName,"content-type:")==0 ){
       cgi_setenv("CONTENT_TYPE", zVal);
@@ -1206,23 +1442,296 @@ void cgi_handle_http_request(const char *zIpAddr){
       cgi_setenv("HTTP_IF_NONE_MATCH", zVal);
     }else if( fossil_strcmp(zFieldName,"if-modified-since:")==0 ){
       cgi_setenv("HTTP_IF_MODIFIED_SINCE", zVal);
-#if 0
     }else if( fossil_strcmp(zFieldName,"referer:")==0 ){
       cgi_setenv("HTTP_REFERER", zVal);
-#endif
     }else if( fossil_strcmp(zFieldName,"user-agent:")==0 ){
       cgi_setenv("HTTP_USER_AGENT", zVal);
+    }else if( fossil_strcmp(zFieldName,"authorization:")==0 ){
+      cgi_setenv("HTTP_AUTHORIZATION", zVal);
+    }else if( fossil_strcmp(zFieldName,"x-forwarded-for:")==0 ){
+      const char *zIpAddr = cgi_accept_forwarded_for(zVal);
+      if( zIpAddr!=0 ){
+        g.zIpAddr = mprintf("%s", zIpAddr);
+        cgi_replace_parameter("REMOTE_ADDR", g.zIpAddr);
+      }
+    }
+  }
+  cgi_init();
+  cgi_trace(0);
+}
+
+/*
+** This routine handles a single HTTP request from an SSH client which is
+** coming in on g.httpIn and which replies on g.httpOut
+**
+** Once all the setup is finished, this procedure returns
+** and subsequent code handles the actual generation of the webpage.
+**
+** It is called in a loop so some variables will need to be replaced
+*/
+void cgi_handle_ssh_http_request(const char *zIpAddr){
+  static int nCycles = 0;
+  static char *zCmd = 0;
+  char *z, *zToken;
+  const char *zType = 0;
+  int i, content_length = 0;
+  char zLine[2000];     /* A single line of input. */
+
+  if( zIpAddr ){
+    if( nCycles==0 ){
+      cgi_setenv("REMOTE_ADDR", zIpAddr);
+      g.zIpAddr = mprintf("%s", zIpAddr);
+    }
+  }else{
+    fossil_panic("missing SSH IP address");
+  }
+  if( fgets(zLine, sizeof(zLine),g.httpIn)==0 ){
+    malformed_request("missing HTTP header");
+  }
+  cgi_trace(zLine);
+  zToken = extract_token(zLine, &z);
+  if( zToken==0 ){
+    malformed_request("malformed HTTP header");
+  }
+
+  if( fossil_strcmp(zToken, "echo")==0 ){
+    /* start looking for probes to complete transport_open */
+    zCmd = cgi_handle_ssh_probes(zLine, sizeof(zLine), z, zToken);
+    if( fgets(zLine, sizeof(zLine),g.httpIn)==0 ){
+      malformed_request("missing HTTP header");
+    }
+    cgi_trace(zLine);
+    zToken = extract_token(zLine, &z);
+    if( zToken==0 ){
+      malformed_request("malformed HTTP header");
+    }
+  }else if( zToken && strlen(zToken)==0 && zCmd ){
+    /* transport_flip request and continued transport_open */
+    cgi_handle_ssh_transport(zCmd);
+    if( fgets(zLine, sizeof(zLine),g.httpIn)==0 ){
+      malformed_request("missing HTTP header");
+    }
+    cgi_trace(zLine);
+    zToken = extract_token(zLine, &z);
+    if( zToken==0 ){
+      malformed_request("malformed HTTP header");
     }
   }
 
+  if( fossil_strcmp(zToken,"GET")!=0 && fossil_strcmp(zToken,"POST")!=0
+      && fossil_strcmp(zToken,"HEAD")!=0 ){
+    malformed_request("unsupported HTTP method");
+  }
+
+  if( nCycles==0 ){
+    cgi_setenv("GATEWAY_INTERFACE","CGI/1.0");
+    cgi_setenv("REQUEST_METHOD",zToken);
+  }
+
+  zToken = extract_token(z, &z);
+  if( zToken==0 ){
+    malformed_request("malformed URL in HTTP header");
+  }
+  if( nCycles==0 ){
+    cgi_setenv("REQUEST_URI", zToken);
+    cgi_setenv("SCRIPT_NAME", "");
+  }
+
+  for(i=0; zToken[i] && zToken[i]!='?'; i++){}
+  if( zToken[i] ) zToken[i++] = 0;
+  if( nCycles==0 ){
+    cgi_setenv("PATH_INFO", zToken);
+  }else{
+    cgi_replace_parameter("PATH_INFO", mprintf("%s",zToken));
+  }
+
+  /* Get all the optional fields that follow the first line.
+  */
+  while( fgets(zLine,sizeof(zLine),g.httpIn) ){
+    char *zFieldName;
+    char *zVal;
+
+    cgi_trace(zLine);
+    zFieldName = extract_token(zLine,&zVal);
+    if( zFieldName==0 || *zFieldName==0 ) break;
+    while( fossil_isspace(*zVal) ){ zVal++; }
+    i = strlen(zVal);
+    while( i>0 && fossil_isspace(zVal[i-1]) ){ i--; }
+    zVal[i] = 0;
+    for(i=0; zFieldName[i]; i++){
+      zFieldName[i] = fossil_tolower(zFieldName[i]);
+    }
+    if( fossil_strcmp(zFieldName,"content-length:")==0 ){
+      content_length = atoi(zVal);
+    }else if( fossil_strcmp(zFieldName,"content-type:")==0 ){
+      g.zContentType = zType = mprintf("%s", zVal);
+    }else if( fossil_strcmp(zFieldName,"host:")==0 ){
+      if( nCycles==0 ){
+        cgi_setenv("HTTP_HOST", zVal);
+      }
+    }else if( fossil_strcmp(zFieldName,"user-agent:")==0 ){
+      if( nCycles==0 ){
+        cgi_setenv("HTTP_USER_AGENT", zVal);
+      }
+    }else if( fossil_strcmp(zFieldName,"x-fossil-transport:")==0 ){
+      if( fossil_strnicmp(zVal, "ssh", 3)==0 ){
+        if( nCycles==0 ){
+          g.fSshClient |= CGI_SSH_FOSSIL;
+          g.fullHttpReply = 0;
+        }
+      }
+    }
+  }
+
+  if( nCycles==0 ){
+    if( ! ( g.fSshClient & CGI_SSH_FOSSIL ) ){
+      /* did not find new fossil ssh transport */
+      g.fSshClient &= ~CGI_SSH_CLIENT;
+      g.fullHttpReply = 1;
+      cgi_replace_parameter("REMOTE_ADDR", "127.0.0.1");
+    }
+  }
+
+  cgi_reset_content();
+  cgi_destination(CGI_BODY);
+
+  if( content_length>0 && zType ){
+    blob_zero(&g.cgiIn);
+    if( fossil_strcmp(zType, "application/x-fossil")==0 ){
+      blob_read_from_channel(&g.cgiIn, g.httpIn, content_length);
+      blob_uncompress(&g.cgiIn, &g.cgiIn);
+    }else if( fossil_strcmp(zType, "application/x-fossil-debug")==0 ){
+      blob_read_from_channel(&g.cgiIn, g.httpIn, content_length);
+    }else if( fossil_strcmp(zType, "application/x-fossil-uncompressed")==0 ){
+      blob_read_from_channel(&g.cgiIn, g.httpIn, content_length);
+    }
+  }
+  cgi_trace(0);
+  nCycles++;
+}
+
+/*
+** This routine handles the old fossil SSH probes
+*/
+char *cgi_handle_ssh_probes(char *zLine, int zSize, char *z, char *zToken){
+  /* Start looking for probes */
+  while( fossil_strcmp(zToken, "echo")==0 ){
+    zToken = extract_token(z, &z);
+    if( zToken==0 ){
+      malformed_request("malformed probe");
+    }
+    if( fossil_strncmp(zToken, "test", 4)==0 ||
+        fossil_strncmp(zToken, "probe-", 6)==0 ){
+      fprintf(g.httpOut, "%s\n", zToken);
+      fflush(g.httpOut);
+    }else{
+      malformed_request("malformed probe");
+    }
+    if( fgets(zLine, zSize, g.httpIn)==0 ){
+      malformed_request("malformed probe");
+    }
+    cgi_trace(zLine);
+    zToken = extract_token(zLine, &z);
+    if( zToken==0 ){
+      malformed_request("malformed probe");
+    }
+  }
+
+  /* Got all probes now first transport_open is completed
+  ** so return the command that was requested
+  */
+  g.fSshClient |= CGI_SSH_COMPAT;
+  return mprintf("%s", zToken);
+}
+
+/*
+** This routine handles the old fossil SSH transport_flip
+** and transport_open communications if detected.
+*/
+void cgi_handle_ssh_transport(const char *zCmd){
+  char *z, *zToken;
+  char zLine[2000];     /* A single line of input. */
+
+  /* look for second newline of transport_flip */
+  if( fgets(zLine, sizeof(zLine),g.httpIn)==0 ){
+    malformed_request("incorrect transport_flip");
+  }
+  cgi_trace(zLine);
+  zToken = extract_token(zLine, &z);
+  if( zToken && strlen(zToken)==0 ){
+    /* look for path to fossil */
+    if( fgets(zLine, sizeof(zLine),g.httpIn)==0 ){
+      if( zCmd==0 ){
+        malformed_request("missing fossil command");
+      }else{
+        /* no new command so exit */
+        fossil_exit(0);
+      }
+    }
+    cgi_trace(zLine);
+    zToken = extract_token(zLine, &z);
+    if( zToken==0 ){
+      malformed_request("malformed fossil command");
+    }
+    /* see if we've seen the command */
+    if( zCmd && zCmd[0] && fossil_strcmp(zToken, zCmd)==0 ){
+      return;
+    }else{
+      malformed_request("transport_open failed");
+    }
+  }else{
+    malformed_request("transport_flip failed");
+  }
+}
+
+/*
+** This routine handles a single SCGI request which is coming in on
+** g.httpIn and which replies on g.httpOut
+**
+** The SCGI request is read from g.httpIn and is used to initialize
+** entries in the cgi_parameter() hash, as if those entries were
+** environment variables.  A call to cgi_init() completes
+** the setup.  Once all the setup is finished, this procedure returns
+** and subsequent code handles the actual generation of the webpage.
+*/
+void cgi_handle_scgi_request(void){
+  char *zHdr;
+  char *zToFree;
+  int nHdr = 0;
+  int nRead;
+  int c, n, m;
+
+  while( (c = fgetc(g.httpIn))!=EOF && fossil_isdigit((char)c) ){
+    nHdr = nHdr*10 + (char)c - '0';
+  }
+  if( nHdr<16 ) malformed_request("SCGI header too short");
+  zToFree = zHdr = fossil_malloc(nHdr);
+  nRead = (int)fread(zHdr, 1, nHdr, g.httpIn);
+  if( nRead<nHdr ) malformed_request("cannot read entire SCGI header");
+  nHdr = nRead;
+  while( nHdr ){
+    for(n=0; n<nHdr && zHdr[n]; n++){}
+    for(m=n+1; m<nHdr && zHdr[m]; m++){}
+    if( m>=nHdr ) malformed_request("SCGI header formatting error");
+    cgi_set_parameter(zHdr, zHdr+n+1);
+    zHdr += m+1;
+    nHdr -= m+1;
+  }
+  fossil_free(zToFree);
+  fgetc(g.httpIn);  /* Read past the "," separating header from content */
   cgi_init();
 }
 
+
 #if INTERFACE
-/* 
+/*
 ** Bitmap values for the flags parameter to cgi_http_server().
 */
 #define HTTP_SERVER_LOCALHOST      0x0001     /* Bind to 127.0.0.1 only */
+#define HTTP_SERVER_SCGI           0x0002     /* SCGI instead of HTTP */
+#define HTTP_SERVER_HAD_REPOSITORY 0x0004     /* Was the repository open? */
+#define HTTP_SERVER_HAD_CHECKOUT   0x0008     /* Was a checkout open? */
+#define HTTP_SERVER_REPOLIST       0x0010     /* Allow repo listing */
 
 #endif /* INTERFACE */
 
@@ -1242,7 +1751,12 @@ void cgi_handle_http_request(const char *zIpAddr){
 ** Return 0 to each child as it runs.  If unable to establish a
 ** listening socket, return non-zero.
 */
-int cgi_http_server(int mnPort, int mxPort, char *zBrowser, int flags){
+int cgi_http_server(
+  int mnPort, int mxPort,   /* Range of TCP ports to try */
+  const char *zBrowser,     /* Run this browser, if not NULL */
+  const char *zIpAddr,      /* Bind to this IP address, if not null */
+  int flags                 /* HTTP_SERVER_* flags */
+){
 #if defined(_WIN32)
   /* Use win32_http_server() instead */
   fossil_exit(1);
@@ -1261,7 +1775,12 @@ int cgi_http_server(int mnPort, int mxPort, char *zBrowser, int flags){
   while( iPort<=mxPort ){
     memset(&inaddr, 0, sizeof(inaddr));
     inaddr.sin_family = AF_INET;
-    if( flags & HTTP_SERVER_LOCALHOST ){
+    if( zIpAddr ){
+      inaddr.sin_addr.s_addr = inet_addr(zIpAddr);
+      if( inaddr.sin_addr.s_addr == (-1) ){
+        fossil_fatal("not a valid IP address: %s", zIpAddr);
+      }
+    }else if( flags & HTTP_SERVER_LOCALHOST ){
       inaddr.sin_addr.s_addr = htonl(INADDR_LOOPBACK);
     }else{
       inaddr.sin_addr.s_addr = htonl(INADDR_ANY);
@@ -1293,12 +1812,22 @@ int cgi_http_server(int mnPort, int mxPort, char *zBrowser, int flags){
   }
   if( iPort>mxPort ) return 1;
   listen(listener,10);
-  if( iPort>mnPort ){
-    fossil_print("Listening for HTTP requests on TCP port %d\n", iPort);
-    fflush(stdout);
-  }
+  fossil_print("Listening for %s requests on TCP port %d\n",
+     (flags & HTTP_SERVER_SCGI)!=0?"SCGI":"HTTP",  iPort);
+  fflush(stdout);
   if( zBrowser ){
-    zBrowser = mprintf(zBrowser, iPort);
+    assert( strstr(zBrowser,"%d")!=0 );
+    zBrowser = mprintf(zBrowser /*works-like:"%d"*/, iPort);
+#if defined(__CYGWIN__)
+    /* On Cygwin, we can do better than "echo" */
+    if( strncmp(zBrowser, "echo ", 5)==0 ){
+      wchar_t *wUrl = fossil_utf8_to_unicode(zBrowser+5);
+      wUrl[wcslen(wUrl)-2] = 0; /* Strip terminating " &" */
+      if( (size_t)ShellExecuteW(0, L"open", wUrl, 0, 0, 1)<33 ){
+        fossil_warning("cannot start browser\n");
+      }
+    }else
+#endif
     if( system(zBrowser)<0 ){
       fossil_warning("cannot start browser: %s\n", zBrowser);
     }
@@ -1330,7 +1859,7 @@ int cgi_http_server(int mnPort, int mxPort, char *zBrowser, int flags){
           close(1);
           fd = dup(connection);
           if( fd!=1 ) nErr++;
-          if( !g.fHttpTrace && !g.fSqlTrace ){
+          if( !g.fAnyTrace ){
             close(2);
             fd = dup(connection);
             if( fd!=2 ) nErr++;
@@ -1345,7 +1874,7 @@ int cgi_http_server(int mnPort, int mxPort, char *zBrowser, int flags){
       nchildren--;
     }
   }
-  /* NOT REACHED */  
+  /* NOT REACHED */
   fossil_exit(1);
 #endif
   /* NOT REACHED */
@@ -1433,7 +1962,7 @@ time_t mkgmtime(struct tm *p){
   isLeapYr = p->tm_year%4==0 && (p->tm_year%100!=0 || (p->tm_year+300)%400==0);
   p->tm_yday = priorDays[p->tm_mon] + p->tm_mday - 1;
   if( isLeapYr && p->tm_mon>1 ) p->tm_yday++;
-  nDay = (p->tm_year-70)*365 + (p->tm_year-69)/4 -p->tm_year/100 + 
+  nDay = (p->tm_year-70)*365 + (p->tm_year-69)/4 -p->tm_year/100 +
          (p->tm_year+300)/400 + p->tm_yday;
   t = ((nDay*24 + p->tm_hour)*60 + p->tm_min)*60 + p->tm_sec;
   return t;
@@ -1453,3 +1982,21 @@ void cgi_modified_since(time_t objectTime){
   cgi_reply();
   fossil_exit(0);
 }
+
+/*
+** Check to see if the remote client is SSH and return
+** its IP or return default
+*/
+const char *cgi_ssh_remote_addr(const char *zDefault){
+  char *zIndex;
+  const char *zSshConn = fossil_getenv("SSH_CONNECTION");
+
+  if( zSshConn && zSshConn[0] ){
+    char *zSshClient = mprintf("%s",zSshConn);
+    if( (zIndex = strchr(zSshClient,' '))!=0 ){
+      zSshClient[zIndex-zSshClient] = '\0';
+      return zSshClient;
+    }
+  }
+  return zDefault;
+}
