@@ -154,6 +154,9 @@ void diff_print_filenames(const char *zLeft, const char *zRight, u64 diffFlags){
 ** The difference is the set of edits needed to transform pFile1 into
 ** zFile2.  The content of pFile1 is in memory.  zFile2 exists on disk.
 **
+** If fSwapDiff is 1, show the set of edits to transform zFile2 into pFile1
+** instead of the opposite.
+**
 ** Use the internal diff logic if zDiffCmd is NULL.  Otherwise call the
 ** command zDiffCmd to do the diffing.
 **
@@ -169,7 +172,8 @@ void diff_file(
   const char *zDiffCmd,     /* Command for comparison */
   const char *zBinGlob,     /* Treat file names matching this as binary */
   int fIncludeBinary,       /* Include binary files for external diff */
-  u64 diffFlags             /* Flags to control the diff */
+  u64 diffFlags,            /* Flags to control the diff */
+  int fSwapDiff             /* Diff from Zfile2 to Pfile1 */
 ){
   if( zDiffCmd==0 ){
     Blob out;                 /* Diff output text */
@@ -196,7 +200,11 @@ void diff_file(
       }
     }else{
       blob_zero(&out);
-      text_diff(pFile1, &file2, &out, 0, diffFlags);
+      if( fSwapDiff ){
+        text_diff(&file2, pFile1, &out, 0, diffFlags);
+      }else{
+        text_diff(pFile1, &file2, &out, 0, diffFlags);
+      }
       if( blob_size(&out) ){
         diff_print_filenames(zName, zName2, diffFlags);
         fossil_print("%s\n", blob_str(&out));
@@ -254,9 +262,15 @@ void diff_file(
     /* Construct the external diff command */
     blob_zero(&cmd);
     blob_appendf(&cmd, "%s ", zDiffCmd);
-    shell_escape(&cmd, blob_str(&nameFile1));
-    blob_append(&cmd, " ", 1);
-    shell_escape(&cmd, zFile2);
+    if( fSwapDiff ){
+      shell_escape(&cmd, zFile2);
+      blob_append(&cmd, " ", 1);
+      shell_escape(&cmd, blob_str(&nameFile1));
+    }else{
+      shell_escape(&cmd, blob_str(&nameFile1));
+      blob_append(&cmd, " ", 1);
+      shell_escape(&cmd, zFile2);
+    }
 
     /* Run the external diff command */
     fossil_system(blob_str(&cmd));
@@ -305,8 +319,10 @@ void diff_file_mem(
     blob_reset(&out);
   }else{
     Blob cmd;
-    char zTemp1[300];
-    char zTemp2[300];
+    Blob temp1;
+    Blob temp2;
+    Blob prefix1;
+    Blob prefix2;
 
     if( !fIncludeBinary ){
       if( isBin1 || isBin2 ){
@@ -324,25 +340,36 @@ void diff_file_mem(
       }
     }
 
+    /* Construct a prefix for the temporary file names */
+    blob_zero(&prefix1);
+    blob_zero(&prefix2);
+    blob_appendf(&prefix1, "%s-v1", zName);
+    blob_appendf(&prefix2, "%s-v2", zName);
+
     /* Construct a temporary file names */
-    file_tempname(sizeof(zTemp1), zTemp1);
-    file_tempname(sizeof(zTemp2), zTemp2);
-    blob_write_to_file(pFile1, zTemp1);
-    blob_write_to_file(pFile2, zTemp2);
+    file_tempname(&temp1, blob_str(&prefix1));
+    file_tempname(&temp2, blob_str(&prefix2));
+    blob_write_to_file(pFile1, blob_str(&temp1));
+    blob_write_to_file(pFile2, blob_str(&temp2));
 
     /* Construct the external diff command */
     blob_zero(&cmd);
     blob_appendf(&cmd, "%s ", zDiffCmd);
-    shell_escape(&cmd, zTemp1);
+    shell_escape(&cmd, blob_str(&temp1));
     blob_append(&cmd, " ", 1);
-    shell_escape(&cmd, zTemp2);
+    shell_escape(&cmd, blob_str(&temp2));
 
     /* Run the external diff command */
     fossil_system(blob_str(&cmd));
 
     /* Delete the temporary file and clean up memory used */
-    file_delete(zTemp1);
-    file_delete(zTemp2);
+    file_delete(blob_str(&temp1));
+    file_delete(blob_str(&temp2));
+
+    blob_reset(&prefix1);
+    blob_reset(&prefix2);
+    blob_reset(&temp1);
+    blob_reset(&temp2);
     blob_reset(&cmd);
   }
 }
@@ -414,6 +441,7 @@ static void diff_against_disk(
     );
   }
   db_prepare(&q, "%s", blob_sql_text(&sql));
+  blob_reset(&sql);
   while( db_step(&q)==SQLITE_ROW ){
     const char *zPathname = db_column_text(&q,0);
     int isDeleted = db_column_int(&q, 1);
@@ -470,7 +498,7 @@ static void diff_against_disk(
       isBin = fIncludeBinary ? 0 : looks_like_binary(&content);
       diff_print_index(zPathname, diffFlags);
       diff_file(&content, isBin, zFullName, zPathname, zDiffCmd,
-                zBinGlob, fIncludeBinary, diffFlags);
+                zBinGlob, fIncludeBinary, diffFlags, 0);
       blob_reset(&content);
     }
     blob_reset(&fname);
@@ -507,7 +535,7 @@ static void diff_against_undo(
     zFullName = mprintf("%s%s", g.zLocalRoot, zFile);
     db_column_blob(&q, 1, &content);
     diff_file(&content, 0, zFullName, zFile,
-              zDiffCmd, zBinGlob, fIncludeBinary, diffFlags);
+              zDiffCmd, zBinGlob, fIncludeBinary, diffFlags, 0);
     fossil_free(zFullName);
     blob_reset(&content);
   }
@@ -737,13 +765,14 @@ void diff_tk(const char *zSubCmd, int firstArg){
 ** diff programs.
 */
 int diff_include_binary_files(void){
-  if( is_truth(find_option("diff-binary", 0, 1)) ){
-    return 1;
+  const char* zArgIncludeBinary = find_option("diff-binary", 0, 1);
+
+  /* Command line argument have priority on settings */
+  if( zArgIncludeBinary ){
+    return is_truth(zArgIncludeBinary);
+  }else{
+    return db_get_boolean("diff-binary", 1);
   }
-  if( db_get_boolean("diff-binary", 1) ){
-    return 1;
-  }
-  return 0;
 }
 
 /*
@@ -799,6 +828,7 @@ const char *diff_get_binary_glob(void){
 **   --branch BRANCH            Show diff of all changes on BRANCH
 **   --brief                    Show filenames only
 **   --checkin VERSION          Show diff of all changes in VERSION
+**   --command PROG             External diff program - overrides "diff-command"
 **   --context|-c N             Use N lines of context
 **   --diff-binary BOOL         Include binary files when using external commands
 **   --exec-abs-paths           Force absolute path names with external commands.
@@ -870,7 +900,8 @@ void diff_cmd(void){
     db_find_and_open_repository(0, 0);
   }
   if( !isInternDiff ){
-    zDiffCmd = diff_command_external(isGDiff);
+    zDiffCmd = find_option("command", 0, 1);
+    if( zDiffCmd==0 ) zDiffCmd = diff_command_external(isGDiff);
   }
   zBinGlob = diff_get_binary_glob();
   fIncludeBinary = diff_include_binary_files();
