@@ -47,6 +47,7 @@
 **   -l|--log             select log mode (the default)
 **   -n|--limit N         Display the first N changes (default unlimited).
 **                        N<=0 means no limit.
+**   --no-dir-symlinks    Disables support for directory symlinks.
 **   --offset P           skip P changes
 **   -p|--print           select print mode
 **   -r|--revision R      print the given revision (or ckout, if none is given)
@@ -126,7 +127,7 @@ void finfo_cmd(void){
 
     file_tree_name(g.argv[2], &fname, 0, 1);
     if( zRevision ){
-      historical_version_of_file(zRevision, blob_str(&fname), &record, 0,0,0,0);
+      historical_blob(zRevision, blob_str(&fname), &record, 1);
     }else{
       int rid = db_int(0, "SELECT rid FROM vfile WHERE pathname=%B %s",
                        &fname, filename_collation());
@@ -251,7 +252,6 @@ void finfo_cmd(void){
 */
 void cat_cmd(void){
   int i;
-  int rc;
   Blob content, fname;
   const char *zRev;
   db_find_and_open_repository(0, 0);
@@ -263,10 +263,7 @@ void cat_cmd(void){
   for(i=2; i<g.argc; i++){
     file_tree_name(g.argv[i], &fname, 0, 1);
     blob_zero(&content);
-    rc = historical_version_of_file(zRev, blob_str(&fname), &content, 0,0,0,2);
-    if( rc==2 ){
-      fossil_fatal("no such file: %s", g.argv[i]);
-    }
+    historical_blob(zRev, blob_str(&fname), &content, 1);
     blob_write_to_file(&content, "-");
     blob_reset(&fname);
     blob_reset(&content);
@@ -333,7 +330,7 @@ void finfo_page(void){
     return;
   }
   if( g.perm.Admin ){
-    style_submenu_element("MLink Table", "mtab", "%R/mlink?name=%t", zFilename);
+    style_submenu_element("MLink Table", "%R/mlink?name=%t", zFilename);
   }
   if( baseCheckin ){
     compute_direct_ancestors(baseCheckin);
@@ -400,7 +397,7 @@ void finfo_page(void){
   blob_zero(&title);
   if( baseCheckin ){
     char *zUuid = db_text(0, "SELECT uuid FROM blob WHERE rid=%d", baseCheckin);
-    char *zLink = 	href("%R/info/%!S", zUuid);
+    char *zLink = href("%R/info/%!S", zUuid);
     if( n>0 ){
       blob_appendf(&title, "First %d ancestors of file ", n);
     }else{
@@ -456,7 +453,7 @@ void finfo_page(void){
     db_bind_int(&qparent, ":fid", frid);
     db_bind_int(&qparent, ":mid", fmid);
     db_bind_int(&qparent, ":fnid", fnid);
-    while( db_step(&qparent)==SQLITE_ROW && nParent<ArraySize(aParent) ){
+    while( db_step(&qparent)==SQLITE_ROW && nParent<count(aParent) ){
       aParent[nParent] = db_column_int(&qparent, 0);
       nParent++;
     }
