@@ -70,17 +70,17 @@ void show_common_info(
       rid
     );
          /* 01234567890123 */
-    fossil_print("%-13s %s %s\n", zUuidName, zUuid, zDate ? zDate : "");
-    free(zUuid);
+    fossil_print("%-13s %.40s %s\n", zUuidName, zUuid, zDate ? zDate : "");
     free(zDate);
-  }
-  if( zUuid && showComment ){
-    zComment = db_text(0,
-      "SELECT coalesce(ecomment,comment) || "
-      "       ' (user: ' || coalesce(euser,user,'?') || ')' "
-      "  FROM event WHERE objid=%d",
-      rid
-    );
+    if( showComment ){
+      zComment = db_text(0,
+        "SELECT coalesce(ecomment,comment) || "
+        "       ' (user: ' || coalesce(euser,user,'?') || ')' "
+        "  FROM event WHERE objid=%d",
+        rid
+      );
+    }
+    free(zUuid);
   }
   if( showFamily ){
     db_prepare(&q, "SELECT uuid, pid, isprim FROM plink JOIN blob ON pid=rid "
@@ -93,7 +93,7 @@ void show_common_info(
         "SELECT datetime(mtime) || ' UTC' FROM event WHERE objid=%d",
         db_column_int(&q, 1)
       );
-      fossil_print("%-13s %s %s\n", zType, zUuid, zDate);
+      fossil_print("%-13s %.40s %s\n", zType, zUuid, zDate);
       free(zDate);
     }
     db_finalize(&q);
@@ -107,7 +107,7 @@ void show_common_info(
         "SELECT datetime(mtime) || ' UTC' FROM event WHERE objid=%d",
         db_column_int(&q, 1)
       );
-      fossil_print("%-13s %s %s\n", zType, zUuid, zDate);
+      fossil_print("%-13s %.40s %s\n", zType, zUuid, zDate);
       free(zDate);
     }
     db_finalize(&q);
@@ -574,6 +574,7 @@ void ci_page(void){
   sideBySide = !is_false(PD("sbs","1"));
   if( db_step(&q1)==SQLITE_ROW ){
     const char *zUuid = db_column_text(&q1, 0);
+    int nUuid = db_column_bytes(&q1, 0);
     char *zEUser, *zEComment;
     const char *zUser;
     const char *zComment;
@@ -595,7 +596,7 @@ void ci_page(void){
     zOrigDate = db_column_text(&q1, 4);
     @ <div class="section">Overview</div>
     @ <table class="label-value">
-    @ <tr><th>SHA1&nbsp;Hash:</th><td>%s(zUuid)
+    @ <tr><th>%s(hname_alg(nUuid)):</th><td>%s(zUuid)
     if( g.perm.Setup ){
       @ (Record ID: %d(rid))
     }
@@ -828,11 +829,9 @@ void winfo_page(void){
   style_header("Update of \"%h\"", pWiki->zWikiTitle);
   zUuid = db_text(0, "SELECT uuid FROM blob WHERE rid=%d", rid);
   zDate = db_text(0, "SELECT datetime(%.17g)", pWiki->rDate);
-  style_submenu_element("Raw", "Raw", "artifact/%s", zUuid);
-  style_submenu_element("History", "History", "whistory?name=%t",
-                        pWiki->zWikiTitle);
-  style_submenu_element("Page", "Page", "wiki?name=%t",
-                        pWiki->zWikiTitle);
+  style_submenu_element("Raw", "artifact/%s", zUuid);
+  style_submenu_element("History", "whistory?name=%t", pWiki->zWikiTitle);
+  style_submenu_element("Page", "wiki?name=%t", pWiki->zWikiTitle);
   login_anonymous_available();
   @ <div class="section">Overview</div>
   @ <p><table class="label-value">
@@ -1055,46 +1054,43 @@ void vdiff_page(void){
   }
   diffFlags = construct_diff_flags(verboseFlag, sideBySide);
   zW = (diffFlags&DIFF_IGNORE_ALLWS)?"&w":"";
-  style_submenu_element("Path","path",
-                        "%R/timeline?me=%T&you=%T", zFrom, zTo);
+  style_submenu_element("Path", "%R/timeline?me=%T&you=%T", zFrom, zTo);
   if( sideBySide || verboseFlag ){
-    style_submenu_element("Hide Diff", "hidediff",
-                          "%R/vdiff?from=%T&to=%T&sbs=0%s%T%s",
+    style_submenu_element("Hide Diff", "%R/vdiff?from=%T&to=%T&sbs=0%s%T%s",
                           zFrom, zTo,
                           zGlob ? "&glob=" : "", zGlob ? zGlob : "", zW);
   }
   if( !sideBySide ){
-    style_submenu_element("Side-by-Side Diff", "sbsdiff",
+    style_submenu_element("Side-by-Side Diff",
                           "%R/vdiff?from=%T&to=%T&sbs=1%s%T%s",
                           zFrom, zTo,
                           zGlob ? "&glob=" : "", zGlob ? zGlob : "", zW);
   }
   if( sideBySide || !verboseFlag ) {
-    style_submenu_element("Unified Diff", "udiff",
+    style_submenu_element("Unified Diff",
                           "%R/vdiff?from=%T&to=%T&sbs=0&v%s%T%s",
                           zFrom, zTo,
                           zGlob ? "&glob=" : "", zGlob ? zGlob : "", zW);
   }
-  style_submenu_element("Invert", "invert",
+  style_submenu_element("Invert",
                         "%R/vdiff?from=%T&to=%T&sbs=%d%s%s%T%s", zTo, zFrom,
                         sideBySide, (verboseFlag && !sideBySide)?"&v":"",
                         zGlob ? "&glob=" : "", zGlob ? zGlob : "", zW);
   if( zGlob ){
-    style_submenu_element("Clear glob", "clearglob",
+    style_submenu_element("Clear glob",
                           "%R/vdiff?from=%T&to=%T&sbs=%d%s%s", zFrom, zTo,
                           sideBySide, (verboseFlag && !sideBySide)?"&v":"", zW);
   }else{
-    style_submenu_element("Patch", "patch",
-                          "%R/vpatch?from=%T&to=%T%s", zFrom, zTo, zW);
+    style_submenu_element("Patch", "%R/vpatch?from=%T&to=%T%s", zFrom, zTo, zW);
   }
   if( sideBySide || verboseFlag ){
     if( *zW ){
-      style_submenu_element("Show Whitespace Differences", "whitespace",
+      style_submenu_element("Show Whitespace Differences",
                             "%R/vdiff?from=%T&to=%T&sbs=%d%s%s%T", zFrom, zTo,
                             sideBySide, (verboseFlag && !sideBySide)?"&v":"",
                             zGlob ? "&glob=" : "", zGlob ? zGlob : "");
     }else{
-      style_submenu_element("Ignore Whitespace", "ignorews",
+      style_submenu_element("Ignore Whitespace",
                             "%R/vdiff?from=%T&to=%T&sbs=%d%s%s%T&w", zFrom, zTo,
                             sideBySide, (verboseFlag && !sideBySide)?"&v":"",
                             zGlob ? "&glob=" : "", zGlob ? zGlob : "");
@@ -1214,7 +1210,7 @@ int object_description(
   char *prevName = 0;
 
   db_prepare(&q,
-    "SELECT filename.name, datetime(event.mtime),"
+    "SELECT filename.name, datetime(event.mtime,toLocal()),"
     "       coalesce(event.ecomment,event.comment),"
     "       coalesce(event.euser,event.user),"
     "       b.uuid, mlink.mperm,"
@@ -1302,7 +1298,7 @@ int object_description(
   free(prevName);
   db_finalize(&q);
   db_prepare(&q,
-    "SELECT substr(tagname, 6, 10000), datetime(event.mtime),"
+    "SELECT substr(tagname, 6, 10000), datetime(event.mtime, toLocal()),"
     "       coalesce(event.euser, event.user)"
     "  FROM tagxref, tag, event"
     " WHERE tagxref.rid=%d"
@@ -1333,7 +1329,7 @@ int object_description(
   db_finalize(&q);
   if( nWiki==0 ){
     db_prepare(&q,
-      "SELECT datetime(mtime), user, comment, type, uuid, tagid"
+      "SELECT datetime(mtime, toLocal()), user, comment, type, uuid, tagid"
       "  FROM event, blob"
       " WHERE event.objid=%d"
       "   AND blob.rid=%d",
@@ -1384,7 +1380,7 @@ int object_description(
     db_finalize(&q);
   }
   db_prepare(&q,
-    "SELECT target, filename, datetime(mtime), user, src"
+    "SELECT target, filename, datetime(mtime, toLocal()), user, src"
     "  FROM attachment"
     " WHERE src=(SELECT uuid FROM blob WHERE rid=%d)"
     " ORDER BY mtime DESC /*sort*/",
@@ -1392,6 +1388,7 @@ int object_description(
   );
   while( db_step(&q)==SQLITE_ROW ){
     const char *zTarget = db_column_text(&q, 0);
+    int nTarget = db_column_bytes(&q, 0);
     const char *zFilename = db_column_text(&q, 1);
     const char *zDate = db_column_text(&q, 2);
     const char *zUser = db_column_text(&q, 3);
@@ -1402,7 +1399,7 @@ int object_description(
       @ Attachment "%h(zFilename)" to
     }
     objType |= OBJTYPE_ATTACHMENT;
-    if( strlen(zTarget)==UUID_SIZE && validate16(zTarget,UUID_SIZE) ){
+    if( nTarget==UUID_SIZE && validate16(zTarget,UUID_SIZE) ){
       if ( db_exists("SELECT 1 FROM tag WHERE tagname='tkt-%q'",
             zTarget)
       ){
@@ -1516,22 +1513,22 @@ void diff_page(void){
   style_header("Diff");
   zW = (diffFlags&DIFF_IGNORE_ALLWS)?"&w":"";
   if( *zW ){
-    style_submenu_element("Show Whitespace Changes", "Show Whitespace Changes",
+    style_submenu_element("Show Whitespace Changes",
                           "%s/fdiff?v1=%T&v2=%T&sbs=%d",
                           g.zTop, P("v1"), P("v2"), sideBySide);
   }else{
-    style_submenu_element("Ignore Whitespace", "Ignore Whitespace",
+    style_submenu_element("Ignore Whitespace",
                           "%s/fdiff?v1=%T&v2=%T&sbs=%d&w",
                           g.zTop, P("v1"), P("v2"), sideBySide);
   }
-  style_submenu_element("Patch", "Patch", "%s/fdiff?v1=%T&v2=%T&patch",
+  style_submenu_element("Patch", "%s/fdiff?v1=%T&v2=%T&patch",
                         g.zTop, P("v1"), P("v2"));
   if( !sideBySide ){
-    style_submenu_element("Side-by-Side Diff", "sbsdiff",
+    style_submenu_element("Side-by-Side Diff",
                           "%s/fdiff?v1=%T&v2=%T&sbs=1%s",
                           g.zTop, P("v1"), P("v2"), zW);
   }else{
-    style_submenu_element("Unified Diff", "udiff",
+    style_submenu_element("Unified Diff",
                           "%s/fdiff?v1=%T&v2=%T&sbs=0%s",
                           g.zTop, P("v1"), P("v2"), zW);
   }
@@ -1560,17 +1557,23 @@ void diff_page(void){
 /*
 ** WEBPAGE: raw
 ** URL: /raw?name=ARTIFACTID&m=TYPE
+** URL: /raw?ci=BRANCH&filename=NAME
 **
 ** Return the uninterpreted content of an artifact.  Used primarily
 ** to view artifacts that are images.
 */
 void rawartifact_page(void){
-  int rid;
+  int rid = 0;
   char *zUuid;
   const char *zMime;
   Blob content;
 
-  rid = name_to_rid_www("name");
+  if( P("ci") && P("filename") ){
+    rid = artifact_from_ci_and_filename(0);
+  }
+  if( rid==0 ){
+    rid = name_to_rid_www("name");
+  }
   login_check_credentials();
   if( !g.perm.Read ){ login_needed(g.anon.Read); return; }
   if( rid==0 ) fossil_redirect_home();
@@ -1674,11 +1677,10 @@ void hexdump_page(void){
   if( g.perm.Admin ){
     const char *zUuid = db_text("", "SELECT uuid FROM blob WHERE rid=%d", rid);
     if( db_exists("SELECT 1 FROM shun WHERE uuid=%Q", zUuid) ){
-      style_submenu_element("Unshun","Unshun","%s/shun?accept=%s&sub=1#delshun",
+      style_submenu_element("Unshun", "%s/shun?accept=%s&sub=1#delshun",
             g.zTop, zUuid);
     }else{
-      style_submenu_element("Shun","Shun", "%s/shun?shun=%s#addshun",
-            g.zTop, zUuid);
+      style_submenu_element("Shun", "%s/shun?shun=%s#addshun", g.zTop, zUuid);
     }
   }
   style_header("Hex Artifact Content");
@@ -1691,8 +1693,8 @@ void hexdump_page(void){
   blob_zero(&downloadName);
   if( P("verbose")!=0 ) objdescFlags |= OBJDESC_DETAIL;
   object_description(rid, objdescFlags, &downloadName);
-  style_submenu_element("Download", "Download",
-        "%s/raw/%T?name=%s", g.zTop, blob_str(&downloadName), zUuid);
+  style_submenu_element("Download", "%s/raw/%T?name=%s",
+        g.zTop, blob_str(&downloadName), zUuid);
   @ <hr />
   content_get(rid, &content);
   @ <blockquote><pre>
@@ -1701,49 +1703,29 @@ void hexdump_page(void){
   style_footer();
 }
 
-/*
-** Attempt to lookup the specified check-in and file name into an rid.
-*/
-int artifact_from_ci_and_filename(
-  const char *zCI,
-  const char *zFilename
-){
-  int cirid;
-  Manifest *pManifest;
-  ManifestFile *pFile;
-
-  if( zCI==0 ) return 0;
-  if( zFilename==0 ) return 0;
-  cirid = name_to_rid(zCI);
-  pManifest = manifest_get(cirid, CFTYPE_MANIFEST, 0);
-  if( pManifest==0 ) return 0;
-  manifest_file_rewind(pManifest);
-  while( (pFile = manifest_file_next(pManifest,0))!=0 ){
-    if( fossil_strcmp(zFilename, pFile->zName)==0 ){
-      int rid = db_int(0, "SELECT rid FROM blob WHERE uuid=%Q", pFile->zUuid);
-      manifest_destroy(pManifest);
-      return rid;
-    }
-  }
-  return 0;
-}
-
 /*
 ** Look for "ci" and "filename" query parameters.  If found, try to
 ** use them to extract the record ID of an artifact for the file.
+**
+** Also look for "fn" as an alias for "filename".  If either "filename"
+** or "fn" is present but "ci" is missing, use "tip" as a default value
+** for "ci".
 */
-int artifact_from_ci_and_filename_www(void){
+int artifact_from_ci_and_filename(HQuery *pUrl){
   const char *zFilename;
   const char *zCI;
   int cirid;
   Manifest *pManifest;
   ManifestFile *pFile;
 
-  zCI = P("ci");
-  if( zCI==0 ) return 0;
   zFilename = P("filename");
-  if( zFilename==0 ) return 0;
-  cirid = name_to_rid_www("ci");
+  if( zFilename==0 ){
+    zFilename = P("fn");
+    if( zFilename==0 ) return 0;
+  }
+  zCI = P("ci");
+  cirid = name_to_typed_rid(zCI ? zCI : "tip", "ci");
+  if( cirid<=0 ) return 0;
   pManifest = manifest_get(cirid, CFTYPE_MANIFEST, 0);
   if( pManifest==0 ) return 0;
   manifest_file_rewind(pManifest);
@@ -1751,9 +1733,14 @@ int artifact_from_ci_and_filename_www(void){
     if( fossil_strcmp(zFilename, pFile->zName)==0 ){
       int rid = db_int(0, "SELECT rid FROM blob WHERE uuid=%Q", pFile->zUuid);
       manifest_destroy(pManifest);
+      if( pUrl ){
+        url_add_parameter(pUrl, "fn", zFilename);
+        if( zCI ) url_add_parameter(pUrl, "ci", zCI);
+      }
       return rid;
     }
   }
+  manifest_destroy(pManifest);
   return 0;
 }
 
@@ -1842,12 +1829,15 @@ void output_text_with_line_numbers(
 
 
 /*
-** WEBPAGE: whatis
 ** WEBPAGE: artifact
+** WEBPAGE: file
+** WEBPAGE: whatis
 **
-** URL: /artifact/SHA1HASH
-** URL: /artifact?ci=CHECKIN&filename=PATH
-** URL: /whatis/SHA1HASH
+** Typical usage:
+**
+**    /artifact/HASH
+**    /whatis/HASH
+**    /file/NAME
 **
 ** Additional query parameters:
 **
@@ -1857,10 +1847,17 @@ void output_text_with_line_numbers(
 **   ln=M-N+Y-Z      - highlight lines M through N and Y through Z (inclusive)
 **   verbose         - show more detail in the description
 **   download        - redirect to the download (artifact page only)
+**   name=SHA1HASH   - Provide the SHA1HASH as a query parameter
+**   filename=NAME   - Show information for content file NAME
+**   fn=NAME         - "fn" is shorthand for "filename"
+**   ci=VERSION      - The specific check-in to use for "filename=".
 **
 ** The /artifact page show the complete content of a file
-** identified by SHA1HASH as preformatted text.  The
-** /whatis page shows only a description of the file.
+** identified by HASH as preformatted text.  The
+** /whatis page shows only a description of the file.  The /file
+** page shows the most recent version of the file or directory
+** called NAME, or a list of the top-level directory if NAME is
+** omitted.
 */
 void artifact_page(void){
   int rid = 0;
@@ -1874,19 +1871,80 @@ void artifact_page(void){
   const char *zUuid;
   u32 objdescFlags = 0;
   int descOnly = fossil_strcmp(g.zPath,"whatis")==0;
+  int isFile = fossil_strcmp(g.zPath,"file")==0;
   const char *zLn = P("ln");
+  const char *zName = P("name");
+  HQuery url;
 
-  if( P("ci") && P("filename") ){
-    rid = artifact_from_ci_and_filename_www();
-  }
+  url_initialize(&url, g.zPath);
+  rid = artifact_from_ci_and_filename(&url);
   if( rid==0 ){
-    rid = name_to_rid_www("name");
+    url_add_parameter(&url, "name", zName);
+    if( isFile ){
+      /* Do a top-level directory listing in /file mode if no argument
+      ** specified */
+      if( zName==0 || zName[0]==0 ){
+        if( P("ci")==0 ) cgi_set_query_parameter("ci","tip");
+        page_tree();
+        return;
+      }
+      /* Look for a single file with the given name */
+      rid = db_int(0,
+         "SELECT fid FROM filename, mlink, event"
+         " WHERE name=%Q"
+         "   AND mlink.fnid=filename.fnid"
+         "   AND event.objid=mlink.mid"
+         " ORDER BY event.mtime DESC LIMIT 1",
+         zName
+      );
+      /* If no file called NAME exists, instead look for a directory
+      ** with that name, and do a directory listing */
+      if( rid==0 ){
+        int nName = (int)strlen(zName);
+        if( nName && zName[nName-1]=='/' ) nName--;
+        if( db_exists(
+           "SELECT 1 FROM filename"
+           " WHERE name GLOB '%.*q/*' AND substr(name,1,%d)=='%.*q/';",
+           nName, zName, nName+1, nName, zName
+        ) ){
+          if( P("ci")==0 ) cgi_set_query_parameter("ci","tip");
+          page_tree();
+          return;
+        }
+      }
+      /* If no file or directory called NAME: issue an error */
+      if( rid==0 ){
+        style_header("No such file");
+        @ File '%h(zName)' does not exist in this repository.
+        style_footer();
+        return;
+      }
+    }else{
+      rid = name_to_rid_www("name");
+    }
   }
 
   login_check_credentials();
   if( !g.perm.Read ){ login_needed(g.anon.Read); return; }
-  if( rid==0 ) fossil_redirect_home();
-  if( descOnly || P("verbose")!=0 ) objdescFlags |= OBJDESC_DETAIL;
+  if( rid==0 ){
+    style_header("No such artifact");
+    @ Artifact '%h(zName)' does not exist in this repository.
+    style_footer();
+    return;
+  }
+  if( descOnly || P("verbose")!=0 ){
+    url_add_parameter(&url, "verbose", "1");
+    objdescFlags |= OBJDESC_DETAIL;
+  }
+  zUuid = db_text("?", "SELECT uuid FROM blob WHERE rid=%d", rid);
+  if( isFile ){
+    @ <h2>Latest version of file '%h(zName)':</h2>
+    style_submenu_element("Artifact", "%R/artifact/%S", zUuid);
+  }else if( g.perm.Setup ){
+    @ <h2>Artifact %s(zUuid) (%d(rid)):</h2>
+  }else{
+    @ <h2>Artifact %s(zUuid):</h2>
+  }
   blob_zero(&downloadName);
   objType = object_description(rid, objdescFlags, &downloadName);
   if( !descOnly && P("download")!=0 ){
@@ -1897,25 +1955,19 @@ void artifact_page(void){
   if( g.perm.Admin ){
     const char *zUuid = db_text("", "SELECT uuid FROM blob WHERE rid=%d", rid);
     if( db_exists("SELECT 1 FROM shun WHERE uuid=%Q", zUuid) ){
-      style_submenu_element("Unshun","Unshun","%s/shun?accept=%s&sub=1#accshun",
+      style_submenu_element("Unshun", "%s/shun?accept=%s&sub=1#accshun",
             g.zTop, zUuid);
     }else{
-      style_submenu_element("Shun","Shun", "%s/shun?shun=%s#addshun",
-            g.zTop, zUuid);
+      style_submenu_element("Shun", "%s/shun?shun=%s#addshun", g.zTop, zUuid);
     }
   }
-  style_header("%s", descOnly ? "Artifact Description" : "Artifact Content");
-  zUuid = db_text("?", "SELECT uuid FROM blob WHERE rid=%d", rid);
-  if( g.perm.Setup ){
-    @ <h2>Artifact %s(zUuid) (%d(rid)):</h2>
-  }else{
-    @ <h2>Artifact %s(zUuid):</h2>
-  }
+  style_header("%s", isFile ? "File Content" :
+                     descOnly ? "Artifact Description" : "Artifact Content");
   if( g.perm.Admin ){
     Stmt q;
     db_prepare(&q,
       "SELECT coalesce(user.login,rcvfrom.uid),"
-      "       datetime(rcvfrom.mtime), rcvfrom.ipaddr"
+      "       datetime(rcvfrom.mtime,toLocal()), rcvfrom.ipaddr"
       "  FROM blob, rcvfrom LEFT JOIN user ON user.uid=rcvfrom.uid"
       " WHERE blob.rid=%d"
       "   AND rcvfrom.rcvid=blob.rcvid;", rid);
@@ -1927,57 +1979,52 @@ void artifact_page(void){
     }
     db_finalize(&q);
   }
-  style_submenu_element("Download", "Download",
-          "%R/raw/%T?name=%s", blob_str(&downloadName), zUuid);
+  style_submenu_element("Download", "%R/raw/%T?name=%s",
+                         blob_str(&downloadName), zUuid);
   if( db_exists("SELECT 1 FROM mlink WHERE fid=%d", rid) ){
-    style_submenu_element("Check-ins Using", "Check-ins Using",
-          "%R/timeline?n=200&uf=%s",zUuid);
+    style_submenu_element("Check-ins Using", "%R/timeline?n=200&uf=%s", zUuid);
   }
   asText = P("txt")!=0;
   zMime = mimetype_from_name(blob_str(&downloadName));
   if( zMime ){
     if( fossil_strcmp(zMime, "text/html")==0 ){
       if( asText ){
-        style_submenu_element("Html", "Html",
-                              "%s/artifact/%s", g.zTop, zUuid);
+        style_submenu_element("Html", "%s", url_render(&url, "txt", 0, 0, 0));
       }else{
         renderAsHtml = 1;
-        style_submenu_element("Text", "Text",
-                              "%s/artifact/%s?txt=1", g.zTop, zUuid);
+        style_submenu_element("Text", "%s", url_render(&url, "txt", "1", 0, 0));
       }
     }else if( fossil_strcmp(zMime, "text/x-fossil-wiki")==0
            || fossil_strcmp(zMime, "text/x-markdown")==0 ){
       if( asText ){
-        style_submenu_element("Wiki", "Wiki",
-                              "%s/artifact/%s", g.zTop, zUuid);
+        style_submenu_element("Wiki", "%s", url_render(&url, "txt", 0, 0, 0));
       }else{
         renderAsWiki = 1;
-        style_submenu_element("Text", "Text",
-                              "%s/artifact/%s?txt=1", g.zTop, zUuid);
+        style_submenu_element("Text", "%s", url_render(&url, "txt", "1", 0, 0));
       }
     }
   }
   if( (objType & (OBJTYPE_WIKI|OBJTYPE_TICKET))!=0 ){
-    style_submenu_element("Parsed", "Parsed", "%R/info/%s", zUuid);
+    style_submenu_element("Parsed", "%R/info/%s", zUuid);
   }
   if( descOnly ){
-    style_submenu_element("Content", "Content", "%R/artifact/%s", zUuid);
+    style_submenu_element("Content", "%R/artifact/%s", zUuid);
   }else{
-    style_submenu_element("Line Numbers", "Line Numbers",
-                          "%R/artifact/%s%s",zUuid,
-                          ((zLn&&*zLn) ? "" : "?txt=1&ln=0"));
+    if( zLn==0 || atoi(zLn)==0 ){
+      style_submenu_checkbox("ln", "Line Numbers", 0);
+    }
     @ <hr />
     content_get(rid, &content);
     if( renderAsWiki ){
       wiki_render_by_mimetype(&content, zMime);
     }else if( renderAsHtml ){
       @ <iframe src="%R/raw/%T(blob_str(&downloadName))?name=%s(zUuid)"
-      @   width="100%%" frameborder="0" marginwidth="0" marginheight="0"
-      @   sandbox="allow-same-origin"
-      @   onload="this.height=this.contentDocument.documentElement.scrollHeight;">
+      @ width="100%%" frameborder="0" marginwidth="0" marginheight="0"
+      @ sandbox="allow-same-origin"
+      @ onload="this.height=this.contentDocument.documentElement.scrollHeight;">
       @ </iframe>
     }else{
-      style_submenu_element("Hex","Hex", "%s/hexdump?name=%s", g.zTop, zUuid);
+      style_submenu_element("Hex", "%s/hexdump?name=%s", g.zTop, zUuid);
       blob_to_utf8_no_bom(&content, 0);
       zMime = mimetype_from_content(&content);
       @ <blockquote>
@@ -1994,8 +2041,7 @@ void artifact_page(void){
       }else if( strncmp(zMime, "image/", 6)==0 ){
         @ <i>(file is %d(blob_size(&content)) bytes of image data)</i><br />
         @ <img src="%R/raw/%s(zUuid)?m=%s(zMime)" />
-        style_submenu_element("Image", "Image",
-                              "%R/raw/%s?m=%s", zUuid, zMime);
+        style_submenu_element("Image", "%R/raw/%s?m=%s", zUuid, zMime);
       }else{
         @ <i>(file is %d(blob_size(&content)) bytes of binary data)</i>
       }
@@ -2027,11 +2073,10 @@ void tinfo_page(void){
   zUuid = db_text("", "SELECT uuid FROM blob WHERE rid=%d", rid);
   if( g.perm.Admin ){
     if( db_exists("SELECT 1 FROM shun WHERE uuid=%Q", zUuid) ){
-      style_submenu_element("Unshun","Unshun", "%s/shun?accept=%s&sub=1#accshun",
+      style_submenu_element("Unshun", "%s/shun?accept=%s&sub=1#accshun",
             g.zTop, zUuid);
     }else{
-      style_submenu_element("Shun","Shun", "%s/shun?shun=%s#addshun",
-            g.zTop, zUuid);
+      style_submenu_element("Shun", "%s/shun?shun=%s#addshun", g.zTop, zUuid);
     }
   }
   pTktChng = manifest_get(rid, CFTYPE_TICKET, 0);
@@ -2062,15 +2107,14 @@ void tinfo_page(void){
       ? db_text("(No title)", "SELECT title FROM ticket WHERE tkt_uuid=%Q", zTktName)
       : 0;
   style_header("Ticket Change Details");
-  style_submenu_element("Raw", "Raw", "%R/artifact/%s", zUuid);
-  style_submenu_element("History", "History", "%R/tkthistory/%s", zTktName);
-  style_submenu_element("Page", "Page", "%R/tktview/%t", zTktName);
-  style_submenu_element("Timeline", "Timeline", "%R/tkttimeline/%t", zTktName);
+  style_submenu_element("Raw", "%R/artifact/%s", zUuid);
+  style_submenu_element("History", "%R/tkthistory/%s", zTktName);
+  style_submenu_element("Page", "%R/tktview/%t", zTktName);
+  style_submenu_element("Timeline", "%R/tkttimeline/%t", zTktName);
   if( P("plaintext") ){
-    style_submenu_element("Formatted", "Formatted", "%R/info/%s", zUuid);
+    style_submenu_element("Formatted", "%R/info/%s", zUuid);
   }else{
-    style_submenu_element("Plaintext", "Plaintext",
-                          "%R/info/%s?plaintext", zUuid);
+    style_submenu_element("Plaintext", "%R/info/%s?plaintext", zUuid);
   }
 
   @ <div class="section">Overview</div>
@@ -2266,7 +2310,7 @@ void render_color_chooser(
 
      { "custom",  "##" },
   };
-  int nColor = sizeof(aColor)/sizeof(aColor[0])-1;
+  int nColor = count(aColor)-1;
   int stdClrFound = 0;
   int i;
 
@@ -2904,7 +2948,7 @@ void ci_amend_cmd(void){
   fClose = find_option("close",0,0)!=0;
   fHide = find_option("hide",0,0)!=0;
   zChngTime = find_option("chngtime",0,1);
-  db_find_and_open_repository(0,0);
+  db_must_be_within_tree();
   user_select();
   verify_all_options();
   if( g.argc<3 || g.argc>=4 ) usage(AMEND_USAGE_STMT);
