@@ -23,72 +23,37 @@
 #include <errno.h>
 
 /*
-** Make changes to the stable part of the schema (the part that is not
-** simply deleted and reconstructed on a rebuild) to bring the schema
-** up to the latest.
+** Update the schema as necessary
 */
-static const char zSchemaUpdates1[] =
-@ -- Index on the delta table
-@ --
-@ CREATE INDEX IF NOT EXISTS delta_i1 ON delta(srcid);
-@
-@ -- Artifacts that should not be processed are identified in the
-@ -- "shun" table.  Artifacts that are control-file forgeries or
-@ -- spam or artifacts whose contents violate administrative policy
-@ -- can be shunned in order to prevent them from contaminating
-@ -- the repository.
-@ --
-@ -- Shunned artifacts do not exist in the blob table.  Hence they
-@ -- have not artifact ID (rid) and we thus must store their full
-@ -- UUID.
-@ --
-@ CREATE TABLE IF NOT EXISTS shun(
-@   uuid UNIQUE,          -- UUID of artifact to be shunned. Canonical form
-@   mtime INTEGER,        -- When added.  Seconds since 1970
-@   scom TEXT             -- Optional text explaining why the shun occurred
-@ );
-@
-@ -- Artifacts that should not be pushed are stored in the "private"
-@ -- table.  
-@ --
-@ CREATE TABLE IF NOT EXISTS private(rid INTEGER PRIMARY KEY);
-@
-@ -- Some ticket content (such as the originators email address or contact
-@ -- information) needs to be obscured to protect privacy.  This is achieved
-@ -- by storing an SHA1 hash of the content.  For display, the hash is
-@ -- mapped back into the original text using this table.  
-@ --
-@ -- This table contains sensitive information and should not be shared
-@ -- with unauthorized users.
-@ --
-@ CREATE TABLE IF NOT EXISTS concealed(
-@   hash TEXT PRIMARY KEY,    -- The SHA1 hash of content
-@   mtime INTEGER,            -- Time created.  Seconds since 1970
-@   content TEXT              -- Content intended to be concealed
-@ );
-;
-static const char zSchemaUpdates2[] =
-@ -- An entry in this table describes a database query that generates a
-@ -- table of tickets.
-@ --
-@ CREATE TABLE IF NOT EXISTS reportfmt(
-@    rn INTEGER PRIMARY KEY,  -- Report number
-@    owner TEXT,              -- Owner of this report format (not used)
-@    title TEXT UNIQUE,       -- Title of this report
-@    mtime INTEGER,           -- Time last modified.  Seconds since 1970
-@    cols TEXT,               -- A color-key specification
-@    sqlcode TEXT             -- An SQL SELECT statement for this report
-@ );
-;
-
 static void rebuild_update_schema(void){
-  int rc;
-  db_multi_exec(zSchemaUpdates1);
-  db_multi_exec(zSchemaUpdates2);
+  /* Verify that the PLINK table has a new column added by the
+  ** 2014-11-28 schema change.  Create it if necessary.  This code
+  ** can be removed in the future, once all users have upgraded to the
+  ** 2014-11-28 or later schema.
+  */
+  if( !db_table_has_column("repository","plink","baseid") ){
+    db_multi_exec(
+      "ALTER TABLE repository.plink ADD COLUMN baseid;"
+    );
+  }
 
-  rc = db_exists("SELECT 1 FROM sqlite_master"
-                 " WHERE name='user' AND sql GLOB '* mtime *'");
-  if( rc==0 ){
+  /* Verify that the MLINK table has the newer columns added by the
+  ** 2015-01-24 schema change.  Create them if necessary.  This code
+  ** can be removed in the future, once all users have upgraded to the
+  ** 2015-01-24 or later schema.
+  */
+  if( !db_table_has_column("repository","mlink","isaux") ){
+    db_begin_transaction();
+    db_multi_exec(
+      "ALTER TABLE repository.mlink ADD COLUMN pmid INTEGER DEFAULT 0;"
+      "ALTER TABLE repository.mlink ADD COLUMN isaux BOOLEAN DEFAULT 0;"
+    );
+    db_end_transaction(0);
+  }
+
+  /* Add the user.mtime column if it is missing. (2011-04-27)
+  */
+  if( !db_table_has_column("repository", "user", "mtime") ){
     db_multi_exec(
       "CREATE TEMP TABLE temp_user AS SELECT * FROM user;"
       "DROP TABLE user;"
@@ -111,18 +76,19 @@ static void rebuild_update_schema(void){
     );
   }
 
-  rc = db_exists("SELECT 1 FROM sqlite_master"
-                 " WHERE name='config' AND sql GLOB '* mtime *'");
-  if( rc==0 ){
+  /* Add the config.mtime column if it is missing.  (2011-04-27)
+  */
+  if( !db_table_has_column("repository", "config", "mtime") ){
     db_multi_exec(
       "ALTER TABLE config ADD COLUMN mtime INTEGER;"
       "UPDATE config SET mtime=now();"
     );
   }
 
-  rc = db_exists("SELECT 1 FROM sqlite_master"
-                 " WHERE name='shun' AND sql GLOB '* mtime *'");
-  if( rc==0 ){
+  /* Add the shun.mtime and shun.scom columns if they are missing.
+  ** (2011-04-27)
+  */
+  if( !db_table_has_column("repository", "shun", "mtime") ){
     db_multi_exec(
       "ALTER TABLE shun ADD COLUMN mtime INTEGER;"
       "ALTER TABLE shun ADD COLUMN scom TEXT;"
@@ -130,14 +96,27 @@ static void rebuild_update_schema(void){
     );
   }
 
-  rc = db_exists("SELECT 1 FROM sqlite_master"
-                 " WHERE name='reportfmt' AND sql GLOB '* mtime *'");
-  if( rc==0 ){
+  /* Add the reportfmt.mtime column if it is missing. (2011-04-27)
+  */
+  if( !db_table_has_column("repository", "reportfmt", "mtime") ){
+    static const char zCreateReportFmtTable[] =
+    @ -- An entry in this table describes a database query that generates a
+    @ -- table of tickets.
+    @ --
+    @ CREATE TABLE IF NOT EXISTS reportfmt(
+    @    rn INTEGER PRIMARY KEY,  -- Report number
+    @    owner TEXT,              -- Owner of this report format (not used)
+    @    title TEXT UNIQUE,       -- Title of this report
+    @    mtime INTEGER,           -- Time last modified.  Seconds since 1970
+    @    cols TEXT,               -- A color-key specification
+    @    sqlcode TEXT             -- An SQL SELECT statement for this report
+    @ );
+    ;
     db_multi_exec(
       "CREATE TEMP TABLE old_fmt AS SELECT * FROM reportfmt;"
       "DROP TABLE reportfmt;"
     );
-    db_multi_exec(zSchemaUpdates2);
+    db_multi_exec("%s", zCreateReportFmtTable/*safe-for-%s*/);
     db_multi_exec(
       "INSERT OR IGNORE INTO reportfmt(rn,owner,title,cols,sqlcode,mtime)"
         " SELECT rn, owner, title, cols, sqlcode, now() FROM old_fmt;"
@@ -147,15 +126,46 @@ static void rebuild_update_schema(void){
     );
   }
 
-  rc = db_exists("SELECT 1 FROM sqlite_master"
-                 " WHERE name='concealed' AND sql GLOB '* mtime *'");
-  if( rc==0 ){
+  /* Add the concealed.mtime column if it is missing. (2011-04-27)
+  */
+  if( !db_table_has_column("repository", "concealed", "mtime") ){
     db_multi_exec(
       "ALTER TABLE concealed ADD COLUMN mtime INTEGER;"
       "UPDATE concealed SET mtime=now();"
     );
   }
-}  
+
+  /* Do the fossil-2.0 updates to the schema.  (2017-02-28)
+  */
+  rebuild_schema_update_2_0();
+}
+
+/*
+** Update the repository schema for Fossil version 2.0.  (2017-02-28)
+**   (1) Change the CHECK constraint on BLOB.UUID so that the length
+**       is greater than or equal to 40, not exactly equal to 40.
+*/
+void rebuild_schema_update_2_0(void){
+  char *z = db_text(0, "SELECT sql FROM repository.sqlite_master WHERE name='blob'");
+  if( z ){
+    /* Search for:  length(uuid)==40
+    **              0123456789 12345   */
+    int i;
+    for(i=10; z[i]; i++){
+      if( z[i]=='=' && strncmp(&z[i-6],"(uuid)==40",10)==0 ){
+        z[i] = '>';
+        db_multi_exec(
+           "PRAGMA writable_schema=ON;"
+           "UPDATE repository.sqlite_master SET sql=%Q WHERE name LIKE 'blob';"
+           "PRAGMA writable_schema=OFF;",
+           z
+        );
+        break;
+      }
+    }
+    fossil_free(z);
+  }
+}
 
 /*
 ** Variables used to store state information about an on-going "rebuild"
@@ -187,7 +197,7 @@ static void percent_complete(int permill){
 /*
 ** Called after each artifact is processed
 */
-static void rebuild_step_done(rid){
+static void rebuild_step_done(int rid){
   /* assert( bag_find(&bagDone, rid)==0 ); */
   bag_insert(&bagDone, rid);
   if( ttyOutput ){
@@ -207,7 +217,7 @@ static void rebuild_step_done(rid){
 ** called to run "fossil deconstruct" instead of the usual
 ** "fossil rebuild".  In that case, instead of rebuilding the
 ** cross-referencing information, write the file content out
-** to the approriate directory.
+** to the appropriate directory.
 **
 ** In both cases, this routine automatically recurses to process
 ** other artifacts that are deltas off of the current artifact.
@@ -229,7 +239,7 @@ static void rebuild_step(int rid, int size, Blob *pBase){
          "UPDATE blob SET size=%d WHERE rid=%d", blob_size(pBase), rid
       );
     }
-  
+
     /* Find all children of artifact rid */
     db_static_prepare(&q1, "SELECT rid FROM delta WHERE srcid=:rid");
     db_bind_int(&q1, ":rid", rid);
@@ -242,7 +252,7 @@ static void rebuild_step(int rid, int size, Blob *pBase){
     }
     nChild = bag_count(&children);
     db_reset(&q1);
-  
+
     /* Crosslink the artifact */
     if( nChild==0 ){
       pUse = pBase;
@@ -252,11 +262,12 @@ static void rebuild_step(int rid, int size, Blob *pBase){
     }
     if( zFNameFormat==0 ){
       /* We are doing "fossil rebuild" */
-      manifest_crosslink(rid, pUse);
+      manifest_crosslink(rid, pUse, MC_NONE);
     }else{
       /* We are doing "fossil deconstruct" */
       char *zUuid = db_text(0, "SELECT uuid FROM blob WHERE rid=%d", rid);
-      char *zFile = mprintf(zFNameFormat, zUuid, zUuid+prefixLength);
+      char *zFile = mprintf(zFNameFormat /*works-like:"%s:%s"*/,
+                            zUuid, zUuid+prefixLength);
       blob_write_to_file(pUse,zFile);
       free(zFile);
       free(zUuid);
@@ -264,7 +275,7 @@ static void rebuild_step(int rid, int size, Blob *pBase){
     }
     assert( blob_is_reset(pUse) );
     rebuild_step_done(rid);
-  
+
     /* Call all children recursively */
     rid = 0;
     for(cid=bag_first(&children), i=1; cid; cid=bag_next(&children, cid), i++){
@@ -319,7 +330,7 @@ static void rebuild_tag_trunk(void){
 }
 
 /*
-** Core function to rebuild the infomration in the derived tables of a
+** Core function to rebuild the information in the derived tables of a
 ** fossil repository from the blobs. This function is shared between
 ** 'rebuild_database' ('rebuild') and 'reconstruct_cmd'
 ** ('reconstruct'), both of which have to regenerate this information
@@ -331,10 +342,10 @@ static void rebuild_tag_trunk(void){
 ** construct a sane repository.
 */
 int rebuild_db(int randomize, int doOut, int doClustering){
-  Stmt s;
+  Stmt s, q;
   int errCnt = 0;
-  char *zTable;
   int incrSize;
+  Blob sql;
 
   bag_init(&bagDone);
   ttyOutput = doOut;
@@ -343,20 +354,24 @@ int rebuild_db(int randomize, int doOut, int doClustering){
     percent_complete(0);
   }
   rebuild_update_schema();
-  for(;;){
-    zTable = db_text(0,
-       "SELECT name FROM sqlite_master /*scan*/"
-       " WHERE type='table'"
-       " AND name NOT IN ('blob','delta','rcvfrom','user',"
-                         "'config','shun','private','reportfmt',"
-                         "'concealed','accesslog')"
-       " AND name NOT GLOB 'sqlite_*'"
-    );
-    if( zTable==0 ) break;
-    db_multi_exec("DROP TABLE %Q", zTable);
-    free(zTable);
+  blob_init(&sql, 0, 0);
+  db_prepare(&q,
+     "SELECT name FROM sqlite_master /*scan*/"
+     " WHERE type='table'"
+     " AND name NOT IN ('admin_log', 'blob','delta','rcvfrom','user','alias',"
+                       "'config','shun','private','reportfmt',"
+                       "'concealed','accesslog','modreq',"
+                       "'purgeevent','purgeitem','unversioned')"
+     " AND name NOT GLOB 'sqlite_*'"
+     " AND name NOT GLOB 'fx_*'"
+  );
+  while( db_step(&q)==SQLITE_ROW ){
+    blob_appendf(&sql, "DROP TABLE \"%w\";\n", db_column_text(&q,0));
   }
-  db_multi_exec(zRepositorySchema2);
+  db_finalize(&q);
+  db_multi_exec("%s", blob_str(&sql)/*safe-for-%s*/);
+  blob_reset(&sql);
+  db_multi_exec("%s", zRepositorySchema2/*safe-for-%s*/);
   ticket_create_table(0);
   shun_artifacts();
 
@@ -371,6 +386,9 @@ int rebuild_db(int randomize, int doOut, int doClustering){
   db_multi_exec(
     "DELETE FROM config WHERE name IN ('remote-code', 'remote-maxid')"
   );
+  db_multi_exec(
+    "UPDATE user SET mtime=strftime('%%s','now') WHERE mtime IS NULL"
+  );
 
   /* The following should be count(*) instead of max(rid). max(rid) is
   ** an adequate approximation, however, and is much faster for large
@@ -413,7 +431,7 @@ int rebuild_db(int randomize, int doOut, int doClustering){
     }
   }
   db_finalize(&s);
-  manifest_crosslink_end();
+  manifest_crosslink_end(MC_NONE);
   rebuild_tag_trunk();
   if( ttyOutput && !g.fQuiet && totalSize>0 ){
     processCnt += incrSize;
@@ -435,7 +453,7 @@ int rebuild_db(int randomize, int doOut, int doClustering){
 ** Attempt to convert more full-text blobs into delta-blobs for
 ** storage efficiency.
 */
-static void extra_deltification(void){
+void extra_deltification(void){
   Stmt q;
   int topid, previd, rid;
   int prevfnid, fnid;
@@ -518,39 +536,60 @@ static void reconstruct_private_table(void){
 ** executable in a way that changes the database schema.
 **
 ** Options:
-**   --cluster     Compute clusters for unclustered artifacts
-**   --compress    Strive to make the database as small as possible
-**   --force       Force the rebuild to complete even if errors are seen
-**   --noverify    Skip the verification of changes to the BLOB table
-**   --pagesize N  Set the database pagesize to N. (512..65536 and power of 2)
-**   --randomize   Scan artifacts in a random order
-**   --vacuum      Run VACUUM on the database after rebuilding
-**   --wal         Set Write-Ahead-Log journalling mode on the database
-**   --stats       Show artifact statistics after rebuilding
+**   --analyze         Run ANALYZE on the database after rebuilding
+**   --cluster         Compute clusters for unclustered artifacts
+**   --compress        Strive to make the database as small as possible
+**   --compress-only   Skip the rebuilding step. Do --compress only
+**   --deanalyze       Remove ANALYZE tables from the database
+**   --force           Force the rebuild to complete even if errors are seen
+**   --ifneeded        Only do the rebuild if it would change the schema version
+**   --index           Always add in the full-text search index
+**   --noverify        Skip the verification of changes to the BLOB table
+**   --noindex         Always omit the full-text search index
+**   --pagesize N      Set the database pagesize to N. (512..65536 and power of 2)
+**   --quiet           Only show output if there are errors
+**   --randomize       Scan artifacts in a random order
+**   --stats           Show artifact statistics after rebuilding
+**   --vacuum          Run VACUUM on the database after rebuilding
+**   --wal             Set Write-Ahead-Log journalling mode on the database
 **
 ** See also: deconstruct, reconstruct
 */
 void rebuild_database(void){
   int forceFlag;
   int randomizeFlag;
-  int errCnt;
+  int errCnt = 0;
   int omitVerify;
   int doClustering;
   const char *zPagesize;
   int newPagesize = 0;
   int activateWal;
   int runVacuum;
+  int runDeanalyze;
+  int runAnalyze;
   int runCompress;
   int showStats;
+  int runReindex;
+  int optNoIndex;
+  int optIndex;
+  int optIfNeeded;
+  int compressOnlyFlag;
 
   omitVerify = find_option("noverify",0,0)!=0;
   forceFlag = find_option("force","f",0)!=0;
   randomizeFlag = find_option("randomize", 0, 0)!=0;
   doClustering = find_option("cluster", 0, 0)!=0;
   runVacuum = find_option("vacuum",0,0)!=0;
+  runDeanalyze = find_option("deanalyze",0,0)!=0;
+  runAnalyze = find_option("analyze",0,0)!=0;
   runCompress = find_option("compress",0,0)!=0;
   zPagesize = find_option("pagesize",0,1);
   showStats = find_option("stats",0,0)!=0;
+  optIndex = find_option("index",0,0)!=0;
+  optNoIndex = find_option("noindex",0,0)!=0;
+  optIfNeeded = find_option("ifneeded",0,0)!=0;
+  compressOnlyFlag = find_option("compress-only",0,0)!=0;
+  if( compressOnlyFlag ) runCompress = runVacuum = 1;
   if( zPagesize ){
     newPagesize = atoi(zPagesize);
     if( newPagesize<512 || newPagesize>65536
@@ -570,14 +609,28 @@ void rebuild_database(void){
     db_close(1);
     db_open_repository(g.zRepositoryName);
   }
+  runReindex = search_index_exists() && !compressOnlyFlag;
+  if( optIndex ) runReindex = 1;
+  if( optNoIndex ) runReindex = 0;
+  if( optIfNeeded && fossil_strcmp(db_get("aux-schema",""),AUX_SCHEMA_MAX)==0 ){
+    return;
+  }
+
+  /* We should be done with options.. */
+  verify_all_options();
+
   db_begin_transaction();
-  ttyOutput = 1;
-  errCnt = rebuild_db(randomizeFlag, 1, doClustering);
-  reconstruct_private_table();
+  if( !compressOnlyFlag ){
+    search_drop_index();
+    ttyOutput = 1;
+    errCnt = rebuild_db(randomizeFlag, 1, doClustering);
+    reconstruct_private_table();
+  }
   db_multi_exec(
-    "REPLACE INTO config(name,value,mtime) VALUES('content-schema','%s',now());"
-    "REPLACE INTO config(name,value,mtime) VALUES('aux-schema','%s',now());",
-    CONTENT_SCHEMA, AUX_SCHEMA
+    "REPLACE INTO config(name,value,mtime) VALUES('content-schema',%Q,now());"
+    "REPLACE INTO config(name,value,mtime) VALUES('aux-schema',%Q,now());"
+    "REPLACE INTO config(name,value,mtime) VALUES('rebuilt',%Q,now());",
+    CONTENT_SCHEMA, AUX_SCHEMA_MAX, get_version()
   );
   if( errCnt && !forceFlag ){
     fossil_print(
@@ -600,6 +653,16 @@ void rebuild_database(void){
       db_multi_exec("PRAGMA page_size=%d", newPagesize);
       runVacuum = 1;
     }
+    if( runDeanalyze ){
+      db_multi_exec("DROP TABLE IF EXISTS sqlite_stat1;"
+                    "DROP TABLE IF EXISTS sqlite_stat3;"
+                    "DROP TABLE IF EXISTS sqlite_stat4;");
+    }
+    if( runAnalyze ){
+      fossil_print("Analyzing the database... "); fflush(stdout);
+      db_multi_exec("ANALYZE;");
+      fossil_print("done\n");
+    }
     if( runVacuum ){
       fossil_print("Vacuuming the database... "); fflush(stdout);
       db_multi_exec("VACUUM");
@@ -609,8 +672,9 @@ void rebuild_database(void){
       db_multi_exec("PRAGMA journal_mode=WAL;");
     }
   }
+  if( runReindex ) search_rebuild_index();
   if( showStats ){
-    static struct { int idx; const char *zLabel; } aStat[] = {
+    static const struct { int idx; const char *zLabel; } aStat[] = {
        { CFTYPE_ANY,       "Artifacts:" },
        { CFTYPE_MANIFEST,  "Manifests:" },
        { CFTYPE_CLUSTER,   "Clusters:" },
@@ -632,7 +696,9 @@ void rebuild_database(void){
 }
 
 /*
-** COMMAND:  test-detach  ?REPOSITORY?
+** COMMAND: test-detach
+**
+** Usage: %fossil test-detach  ?REPOSITORY?
 **
 ** Change the project-code and make other changes in order to prevent
 ** the repository from ever again pushing or pulling to other
@@ -653,7 +719,7 @@ void test_detach_cmd(void){
 }
 
 /*
-** COMMAND:  test-create-clusters
+** COMMAND: test-create-clusters
 **
 ** Create clusters for all unclustered artifacts if the number of unclustered
 ** artifacts exceeds the current clustering threshold.
@@ -671,7 +737,7 @@ void test_createcluster_cmd(void){
   }
   db_begin_transaction();
   create_cluster();
-  db_end_transaction(0);  
+  db_end_transaction(0);
 }
 
 /*
@@ -684,7 +750,7 @@ void test_clusters_cmd(void){
   Bag pending;
   Stmt q;
   int n;
-  
+
   db_find_and_open_repository(0, 2);
   bag_init(&pending);
   db_multi_exec(
@@ -706,9 +772,9 @@ void test_clusters_cmd(void){
     Manifest *p;
     int rid = bag_first(&pending);
     int i;
-    
+
     bag_remove(&pending, rid);
-    p = manifest_get(rid, CFTYPE_CLUSTER);
+    p = manifest_get(rid, CFTYPE_CLUSTER, 0);
     if( p==0 ){
       fossil_fatal("bad cluster: rid=%d", rid);
     }
@@ -744,7 +810,8 @@ void test_clusters_cmd(void){
 
 /*
 ** COMMAND: scrub*
-** %fossil scrub ?OPTIONS? ?REPOSITORY?
+**
+** Usage: %fossil scrub ?OPTIONS? ?REPOSITORY?
 **
 ** The command removes sensitive information (such as passwords) from a
 ** repository so that the repository can be sent to an untrusted reader.
@@ -772,13 +839,20 @@ void scrub_cmd(void){
   int privateOnly = find_option("private",0,0)!=0;
   int bNeedRebuild = 0;
   db_find_and_open_repository(OPEN_ANY_SCHEMA, 2);
+  db_close(1);
+  db_open_repository(g.zRepositoryName);
+
+  /* We should be done with options.. */
+  verify_all_options();
+
   if( !bForce ){
     Blob ans;
-    blob_zero(&ans);
+    char cReply;
     prompt_user(
          "Scrubbing the repository will permanently delete information.\n"
          "Changes cannot be undone.  Continue (y/N)? ", &ans);
-    if( blob_str(&ans)[0]!='y' ){
+    cReply = blob_str(&ans)[0];
+    if( cReply!='y' && cReply!='Y' ){
       fossil_exit(1);
     }
   }
@@ -798,10 +872,14 @@ void scrub_cmd(void){
     );
     if( bVerily ){
       db_multi_exec(
-        "DELETE FROM concealed;"
-        "UPDATE rcvfrom SET ipaddr='unknown';"
-        "DROP TABLE IF EXISTS accesslog;"
-        "UPDATE user SET photo=NULL, info='';"
+        "DELETE FROM concealed;\n"
+        "UPDATE rcvfrom SET ipaddr='unknown';\n"
+        "DROP TABLE IF EXISTS accesslog;\n"
+        "UPDATE user SET photo=NULL, info='';\n"
+        "DROP TABLE IF EXISTS purgeevent;\n"
+        "DROP TABLE IF EXISTS purgeitem;\n"
+        "DROP TABLE IF EXISTS admin_log;\n"
+        "DROP TABLE IF EXISTS vcache;\n"
       );
     }
   }
@@ -826,7 +904,7 @@ void recon_read_dir(char *zPath){
   void *zUnicodePath;
   char *zUtf8Name;
 
-  zUnicodePath = fossil_utf8_to_unicode(zPath);
+  zUnicodePath = fossil_utf8_to_path(zPath, 1);
   d = opendir(zUnicodePath);
   if( d ){
     while( (pEntry=readdir(d))!=0 ){
@@ -836,31 +914,38 @@ void recon_read_dir(char *zPath){
       if( pEntry->d_name[0]=='.' ){
         continue;
       }
-      zUtf8Name = fossil_unicode_to_utf8(pEntry->d_name);
+      zUtf8Name = fossil_path_to_utf8(pEntry->d_name);
       zSubpath = mprintf("%s/%s", zPath, zUtf8Name);
-      fossil_mbcs_free(zUtf8Name);
-      if( file_isdir(zSubpath)==1 ){
+      fossil_path_free(zUtf8Name);
+#ifdef _DIRENT_HAVE_D_TYPE
+      if( (pEntry->d_type==DT_UNKNOWN || pEntry->d_type==DT_LNK)
+          ? (file_isdir(zSubpath)==1) : (pEntry->d_type==DT_DIR) )
+#else
+      if( file_isdir(zSubpath)==1 )
+#endif
+      {
         recon_read_dir(zSubpath);
+      }else{
+        blob_init(&path, 0, 0);
+        blob_appendf(&path, "%s", zSubpath);
+        if( blob_read_from_file(&aContent, blob_str(&path))==-1 ){
+          fossil_fatal("some unknown error occurred while reading \"%s\"",
+                       blob_str(&path));
+        }
+        content_put(&aContent);
+        blob_reset(&path);
+        blob_reset(&aContent);
+        fossil_print("\r%d", ++nFileRead);
+        fflush(stdout);
       }
-      blob_init(&path, 0, 0);
-      blob_appendf(&path, "%s", zSubpath);
-      if( blob_read_from_file(&aContent, blob_str(&path))==-1 ){
-        fossil_panic("some unknown error occurred while reading \"%s\"", 
-                     blob_str(&path));
-      }
-      content_put(&aContent);
-      blob_reset(&path);
-      blob_reset(&aContent);
       free(zSubpath);
-      fossil_print("\r%d", ++nFileRead);
-      fflush(stdout);
     }
     closedir(d);
   }else {
-    fossil_panic("encountered error %d while trying to open \"%s\".",
+    fossil_fatal("encountered error %d while trying to open \"%s\".",
                   errno, g.argv[3]);
   }
-  fossil_mbcs_free(zUnicodePath);
+  fossil_path_free(zUnicodePath);
 }
 
 /*
@@ -886,9 +971,13 @@ void reconstruct_cmd(void) {
   }
   db_create_repository(g.argv[2]);
   db_open_repository(g.argv[2]);
-  db_open_config(0);
+
+  /* We should be done with options.. */
+  verify_all_options();
+
+  db_open_config(0, 0);
   db_begin_transaction();
-  db_initial_setup(0, 0, 0, 1);
+  db_initial_setup(0, 0, 0);
 
   fossil_print("Reading files from directory \"%s\"...\n", g.argv[3]);
   recon_read_dir(g.argv[3]);
@@ -902,7 +991,7 @@ void reconstruct_cmd(void) {
   ** long time.
   */
   verify_cancel();
-  
+
   db_end_transaction(0);
   fossil_print("project-id: %s\n", db_get("project-code", 0));
   fossil_print("server-id: %s\n", db_get("server-code", 0));
@@ -919,10 +1008,10 @@ void reconstruct_cmd(void) {
 ** This command exports all artifacts of a given repository and
 ** writes all artifacts to the file system. The DESTINATION directory
 ** will be populated with subdirectories AA and files AA/BBBBBBBBB.., where
-** AABBBBBBBBB.. is the 40 character artifact ID, AA the first 2 characters.
+** AABBBBBBBBB.. is the 40+ character artifact ID, AA the first 2 characters.
 ** If -L|--prefixlength is given, the length (default 2) of the directory
 ** prefix can be set to 0,1,..,9 characters.
-** 
+**
 ** Options:
 **   -R|--repository REPOSITORY  deconstruct given REPOSITORY
 **   -L|--prefixlength N         set the length of the names of the DESTINATION
@@ -945,7 +1034,7 @@ void deconstruct_cmd(void){
     if( zPrefixOpt[0]>='0' && zPrefixOpt[0]<='9' && !zPrefixOpt[1] ){
       prefixLength = (int)(*zPrefixOpt-'0');
     }else{
-      fossil_fatal("N(%s) is not a a valid prefix length!",zPrefixOpt);
+      fossil_fatal("N(%s) is not a valid prefix length!",zPrefixOpt);
     }
   }
   /* open repository and open query for all artifacts */
@@ -967,7 +1056,7 @@ void deconstruct_cmd(void){
   }
 #else
   /* write access on windows is not checked, errors will be
-  ** dected on blob_write_to_file
+  ** detected on blob_write_to_file
   */
 #endif
   if( prefixLength ){
