@@ -29,6 +29,10 @@
 #  include <zlib.h>
 #endif
 
+#ifndef _WIN32
+#  include "linenoise.h"
+#endif
+
 /*
 ** Implementation of the "content(X)" SQL function.  Return the complete
 ** content of artifact identified by X as a blob.
@@ -147,31 +151,44 @@ static int sqlcmd_autoinit(
   foci_register(db);
   g.repositoryOpen = 1;
   g.db = db;
+  sqlite3_db_config(db, SQLITE_DBCONFIG_MAINDBNAME, "repository");
+  if( g.zLocalDbName ){
+    char *zSql = sqlite3_mprintf("ATTACH %Q AS 'localdb'", g.zLocalDbName);
+    sqlite3_exec(db, zSql, 0, 0, 0);
+    sqlite3_free(zSql);
+  }
+  if( g.zConfigDbName ){
+    char *zSql = sqlite3_mprintf("ATTACH %Q AS 'configdb'", g.zConfigDbName);
+    sqlite3_exec(db, zSql, 0, 0, 0);
+    sqlite3_free(zSql);
+  }
   return SQLITE_OK;
 }
 
 /*
 ** COMMAND: sqlite3
 **
-** Usage: %fossil sqlite3 ?FOSSIL_OPTS? ?DATABASE? ?SHELL_OPTS?
+** Usage: %fossil sql ?OPTIONS?
 **
 ** Run the standalone sqlite3 command-line shell on DATABASE with SHELL_OPTS.
 ** If DATABASE is omitted, then the repository that serves the working
 ** directory is opened.  See https://www.sqlite.org/cli.html for additional
 ** information.
 **
-** Fossil Options:
+** Options:
 **
 **    --no-repository           Skip opening the repository database.
 **
+**    -R REPOSITORY             Use REPOSITORY as the repository database
+**
 ** WARNING:  Careless use of this command can corrupt a Fossil repository
 ** in ways that are unrecoverable.  Be sure you know what you are doing before
 ** running any SQL commands that modify the repository database.
 **
 ** The following extensions to the usual SQLite commands are provided:
 **
-**    content(X)                Return the content of artifact X.  X can be a
-**                              SHA1 hash or prefix or a tag.
+**    content(X)                Return the content of artifact X.  X can be an
+**                              artifact hash or prefix or a tag.
 **
 **    compress(X)               Compress text X.
 **
@@ -195,13 +212,23 @@ static int sqlcmd_autoinit(
 */
 void cmd_sqlite3(void){
   int noRepository;
+  const char *zConfigDb;
   extern int sqlite3_shell(int, char**);
+#ifdef FOSSIL_ENABLE_TH1_HOOKS
+  g.fNoThHook = 1;
+#endif
   noRepository = find_option("no-repository", 0, 0)!=0;
   if( !noRepository ){
     db_find_and_open_repository(OPEN_ANY_SCHEMA, 0);
   }
+  db_open_config(1,0);
+  zConfigDb = g.zConfigDbName;
   fossil_close(1, noRepository);
   sqlite3_shutdown();
+#ifndef _WIN32
+  linenoiseSetMultiLine(1);
+#endif
+  g.zConfigDbName = zConfigDb;
   sqlite3_shell(g.argc-1, g.argv+1);
   sqlite3_cancel_auto_extension((void(*)(void))sqlcmd_autoinit);
   fossil_close(0, noRepository);
