@@ -17,9 +17,14 @@
 **
 ** This file contains code used to generate tarballs.
 */
-#include <assert.h>
-#include <zlib.h>
 #include "config.h"
+#include <assert.h>
+#if defined(FOSSIL_ENABLE_MINIZ)
+#  define MINIZ_HEADER_FILE_ONLY
+#  include "miniz.c"
+#else
+#  include <zlib.h>
+#endif
 #include "tar.h"
 
 /*
@@ -70,7 +75,7 @@ static void tar_begin(sqlite3_int64 mTime){
 
 
 /*
-** verify that lla characters in 'zName' are in the
+** Verify that all characters in 'zName' are in the
 ** ISO646 (=ASCII) character set.
 */
 static int is_iso646_name(
@@ -87,7 +92,7 @@ static int is_iso646_name(
 
 
 /*
-**   copy string pSrc into pDst, truncating or padding with 0 if necessary
+**  copy string pSrc into pDst, truncating or padding with 0 if necessary
 */
 static void padded_copy(
   char *pDest,
@@ -282,7 +287,7 @@ static void tar_add_header(
   int iMode,             /* Mode.  0644 or 0755 */
   unsigned int mTime,    /* File modification time */
   int iSize,             /* Size of the object in bytes */
-  char cType             /* Type of object:  
+  char cType             /* Type of object:
                             '0'==file. '2'==symlink. '5'==directory */
 ){
   /* set mode and modification time */
@@ -338,8 +343,9 @@ static void tar_add_directory_of(
   int i;
   for(i=nName-1; i>0 && zName[i]!='/'; i--){}
   if( i<=0 ) return;
-  if( i < tball.nPrevDirAlloc && tball.zPrevDir[i]==0 &&
-        memcmp(tball.zPrevDir, zName, i)==0 ) return;
+  if( i<tball.nPrevDirAlloc
+   && strncmp(tball.zPrevDir, zName, i)==0
+   && tball.zPrevDir[i]==0 ) return;
   db_multi_exec("INSERT OR IGNORE INTO dir VALUES('%#q')", i, zName);
   if( sqlite3_changes(g.db)==0 ) return;
   tar_add_directory_of(zName, i-1, mTime);
@@ -373,7 +379,7 @@ static void tar_add_file(
   /* length check moved to tar_split_path */
   tar_add_directory_of(zName, nName, mTime);
 
-  /* 
+  /*
    * If we have a symlink, write its destination path (which is stored in
    * pContent) into header, and set content length to 0 to avoid storing path
    * as file content in the next step.  Since 'linkname' header is limited to
@@ -386,7 +392,7 @@ static void tar_add_file(
     n = 0;
   }
 
-  tar_add_header(zName, nName, ( mPerm==PERM_EXE ) ? 0755 : 0644, 
+  tar_add_header(zName, nName, ( mPerm==PERM_EXE ) ? 0755 : 0644,
                  mTime, n, cType);
   if( n ){
     gzip_step(blob_buffer(pContent), n);
@@ -424,17 +430,16 @@ static void tar_finish(Blob *pOut){
 void test_tarball_cmd(void){
   int i;
   Blob zip;
-  Blob file;
   if( g.argc<3 ){
     usage("ARCHIVE FILE....");
   }
   sqlite3_open(":memory:", &g.db);
-  tar_begin(0);
+  tar_begin(-1);
   for(i=3; i<g.argc; i++){
+    Blob file;
     blob_zero(&file);
     blob_read_from_file(&file, g.argv[i]);
-    tar_add_file(g.argv[i], &file,
-                 file_wd_perm(g.argv[i]), file_wd_mtime(g.argv[i]));
+    tar_add_file(g.argv[i], &file, file_wd_perm(0), file_wd_mtime(0));
     blob_reset(&file);
   }
   tar_finish(&zip);
@@ -442,12 +447,13 @@ void test_tarball_cmd(void){
 }
 
 /*
-** Given the RID for a checkin, construct a tarball containing
-** all files in that checkin
+** Given the RID for a check-in, construct a tarball containing
+** all files in that check-in that match pGlob (or all files if
+** pGlob is NULL).
 **
 ** If RID is for an object that is not a real manifest, then the
 ** resulting tarball contains a single file which is the RID
-** object.
+** object.  pInclude and pExclude are ignored in this case.
 **
 ** If the RID object does not exist in the repository, then
 ** pTar is zeroed.
@@ -459,13 +465,19 @@ void test_tarball_cmd(void){
 ** with source files. For example, pass a UUID or "ProjectName".
 **
 */
-void tarball_of_checkin(int rid, Blob *pTar, const char *zDir){
+void tarball_of_checkin(
+  int rid,             /* The RID of the checkin from which to form a tarball */
+  Blob *pTar,          /* Write the tarball into this blob */
+  const char *zDir,    /* Directory prefix for all file added to tarball */
+  Glob *pInclude,      /* Only add files matching this pattern */
+  Glob *pExclude       /* Exclude files matching this pattern */
+){
   Blob mfile, hash, file;
   Manifest *pManifest;
   ManifestFile *pFile;
   Blob filename;
   int nPrefix;
-  char *zName;
+  char *zName = 0;
   unsigned int mTime;
 
   content_get(rid, &mfile);
@@ -473,7 +485,7 @@ void tarball_of_checkin(int rid, Blob *pTar, const char *zDir){
     blob_zero(pTar);
     return;
   }
-  blob_zero(&hash);
+  blob_set_dynamic(&hash, rid_to_uuid(rid));
   blob_zero(&filename);
 
   if( zDir && zDir[0] ){
@@ -481,26 +493,65 @@ void tarball_of_checkin(int rid, Blob *pTar, const char *zDir){
   }
   nPrefix = blob_size(&filename);
 
-  pManifest = manifest_get(rid, CFTYPE_MANIFEST);
+  pManifest = manifest_get(rid, CFTYPE_MANIFEST, 0);
   if( pManifest ){
+    int flg, eflg = 0;
     mTime = (pManifest->rDate - 2440587.5)*86400.0;
     tar_begin(mTime);
-    if( db_get_boolean("manifest", 0) ){
-      blob_append(&filename, "manifest", -1);
-      zName = blob_str(&filename);
-      tar_add_file(zName, &mfile, 0, mTime);
-      sha1sum_blob(&mfile, &hash);
+    flg = db_get_manifest_setting();
+    if( flg ){
+      /* eflg is the effective flags, taking include/exclude into account */
+      if( (pInclude==0 || glob_match(pInclude, "manifest"))
+       && !glob_match(pExclude, "manifest")
+       && (flg & MFESTFLG_RAW) ){
+        eflg |= MFESTFLG_RAW;
+      }
+      if( (pInclude==0 || glob_match(pInclude, "manifest.uuid"))
+       && !glob_match(pExclude, "manifest.uuid")
+       && (flg & MFESTFLG_UUID) ){
+        eflg |= MFESTFLG_UUID;
+      }
+      if( (pInclude==0 || glob_match(pInclude, "manifest.tags"))
+       && !glob_match(pExclude, "manifest.tags")
+       && (flg & MFESTFLG_TAGS) ){
+        eflg |= MFESTFLG_TAGS;
+      }
+
+      if( eflg & (MFESTFLG_RAW|MFESTFLG_UUID) ){
+        if( eflg & MFESTFLG_RAW ){
+          blob_append(&filename, "manifest", -1);
+          zName = blob_str(&filename);
+        }
+        if( eflg & MFESTFLG_RAW ) {
+          sterilize_manifest(&mfile);
+          tar_add_file(zName, &mfile, 0, mTime);
+        }
+      }
       blob_reset(&mfile);
-      blob_append(&hash, "\n", 1);
-      blob_resize(&filename, nPrefix);
-      blob_append(&filename, "manifest.uuid", -1);
-      zName = blob_str(&filename);
-      tar_add_file(zName, &hash, 0, mTime);
-      blob_reset(&hash);
+      if( eflg & MFESTFLG_UUID ){
+        blob_append(&hash, "\n", 1);
+        blob_resize(&filename, nPrefix);
+        blob_append(&filename, "manifest.uuid", -1);
+        zName = blob_str(&filename);
+        tar_add_file(zName, &hash, 0, mTime);
+      }
+      if( eflg & MFESTFLG_TAGS ){
+        Blob tagslist;
+        blob_zero(&tagslist);
+        get_checkin_taglist(rid, &tagslist);
+        blob_resize(&filename, nPrefix);
+        blob_append(&filename, "manifest.tags", -1);
+        zName = blob_str(&filename);
+        tar_add_file(zName, &tagslist, 0, mTime);
+        blob_reset(&tagslist);
+      }
     }
     manifest_file_rewind(pManifest);
     while( (pFile = manifest_file_next(pManifest,0))!=0 ){
-      int fid = uuid_to_rid(pFile->zUuid, 0);
+      int fid;
+      if( pInclude!=0 && !glob_match(pInclude, pFile->zName) ) continue;
+      if( glob_match(pExclude, pFile->zName) ) continue;
+      fid = uuid_to_rid(pFile->zUuid, 0);
       if( fid ){
         content_get(fid, &file);
         blob_resize(&filename, nPrefix);
@@ -511,7 +562,6 @@ void tarball_of_checkin(int rid, Blob *pTar, const char *zDir){
       }
     }
   }else{
-    sha1sum_blob(&mfile, &hash);
     blob_append(&filename, blob_str(&hash), 16);
     zName = blob_str(&filename);
     mTime = db_int64(0, "SELECT (julianday('now') -  2440587.5)*86400.0;");
@@ -520,6 +570,7 @@ void tarball_of_checkin(int rid, Blob *pTar, const char *zDir){
   }
   manifest_destroy(pManifest);
   blob_reset(&mfile);
+  blob_reset(&hash);
   blob_reset(&filename);
   tar_finish(pTar);
 }
@@ -527,24 +578,52 @@ void tarball_of_checkin(int rid, Blob *pTar, const char *zDir){
 /*
 ** COMMAND: tarball*
 **
-** Usage: %fossil tarball VERSION OUTPUTFILE [--name DIRECTORYNAME] [-R|--repository REPO]
+** Usage: %fossil tarball VERSION OUTPUTFILE [OPTIONS]
 **
 ** Generate a compressed tarball for a specified version.  If the --name
 ** option is used, its argument becomes the name of the top-level directory
 ** in the resulting tarball.  If --name is omitted, the top-level directory
-** named is derived from the project name, the check-in date and time, and
+** name is derived from the project name, the check-in date and time, and
 ** the artifact ID of the check-in.
+**
+** The GLOBLIST argument to --exclude and --include can be a comma-separated
+** list of glob patterns, where each glob pattern may optionally be enclosed
+** in "..." or '...' so that it may contain commas.  If a file matches both
+** --include and --exclude then it is excluded.
+**
+** Options:
+**   -X|--exclude GLOBLIST   Comma-separated list of GLOBs of files to exclude
+**   --include GLOBLIST      Comma-separated list of GLOBs of files to include
+**   --name DIRECTORYNAME    The name of the top-level directory in the archive
+**   -R REPOSITORY           Specify a Fossil repository
 */
 void tarball_cmd(void){
   int rid;
   Blob tarball;
   const char *zName;
+  Glob *pInclude = 0;
+  Glob *pExclude = 0;
+  const char *zInclude;
+  const char *zExclude;
   zName = find_option("name", 0, 1);
+  zExclude = find_option("exclude", "X", 1);
+  if( zExclude ) pExclude = glob_create(zExclude);
+  zInclude = find_option("include", 0, 1);
+  if( zInclude ) pInclude = glob_create(zInclude);
   db_find_and_open_repository(0, 0);
+
+  /* We should be done with options.. */
+  verify_all_options();
+
   if( g.argc!=4 ){
     usage("VERSION OUTPUTFILE");
   }
   rid = name_to_typed_rid(g.argv[2], "ci");
+  if( rid==0 ){
+    fossil_fatal("Check-in not found: %s", g.argv[2]);
+    return;
+  }
+
   if( zName==0 ){
     zName = db_text("default-name",
        "SELECT replace(%Q,' ','_') "
@@ -556,30 +635,68 @@ void tarball_cmd(void){
        db_get("project-name", "unnamed"), rid, rid
     );
   }
-  tarball_of_checkin(rid, &tarball, zName);
+  tarball_of_checkin(rid, &tarball, zName, pInclude, pExclude);
+  glob_free(pInclude);
+  glob_free(pExclude);
   blob_write_to_file(&tarball, g.argv[3]);
   blob_reset(&tarball);
 }
 
 /*
 ** WEBPAGE: tarball
-** URL: /tarball/RID.tar.gz
+** URL: /tarball
 **
-** Generate a compressed tarball for a checkin.
-** Return that tarball as the HTTP reply content.
+** Generate a compressed tarball for the check-in specified by the "r"
+** query parameter.  Return that compressed tarball as the HTTP reply
+** content.
+**
+** Query parameters:
+**
+**   name=NAME[.tar.gz]  The base name of the output file.  The default
+**                       value is a configuration parameter in the project
+**                       settings.  A prefix of the name, omitting the
+**                       extension, is used as the top-most directory name.
+**
+**   r=TAG               The check-in that is turned into a compressed tarball.
+**                       Defaults to "trunk".  This query parameter used to
+**                       be called "uuid" and "uuid" is still accepted for
+**                       backwards compatibility.  If omitted, the default
+**                       check-in name is "trunk".
+**
+**   in=PATTERN          Only include files that match the comma-separate
+**                       list of GLOB patterns in PATTERN, as with ex=
+**
+**   ex=PATTERN          Omit any file that match PATTERN.  PATTERN is a
+**                       comma-separated list of GLOB patterns, where each
+**                       pattern can optionally be quoted using ".." or '..'.
+**                       Any file matching both ex= and in= is excluded.
 */
 void tarball_page(void){
   int rid;
-  char *zName, *zRid;
+  char *zName, *zRid, *zKey;
   int nName, nRid;
-  Blob tarball;
+  const char *zInclude;         /* The in= query parameter */
+  const char *zExclude;         /* The ex= query parameter */
+  Blob cacheKey;                /* The key to cache */
+  Glob *pInclude = 0;           /* The compiled in= glob pattern */
+  Glob *pExclude = 0;           /* The compiled ex= glob pattern */
+  Blob tarball;                 /* Tarball accumulated here */
+  const char *z;
 
   login_check_credentials();
-  if( !g.perm.Zip ){ login_needed(); return; }
+  if( !g.perm.Zip ){ login_needed(g.anon.Zip); return; }
+  load_control();
   zName = mprintf("%s", PD("name",""));
   nName = strlen(zName);
-  zRid = mprintf("%s", PD("uuid",""));
+  z = P("r");
+  if( z==0 ) z = P("uuid");
+  if( z==0 ) z = "trunk";
+  zRid = fossil_strdup(z);
   nRid = strlen(zRid);
+  zInclude = P("in");
+  if( zInclude ) pInclude = glob_create(zInclude);
+  zExclude = P("ex");
+  if( zExclude ) pExclude = glob_create(zExclude);
   if( nName>7 && fossil_strcmp(&zName[nName-7], ".tar.gz")==0 ){
     /* Special case:  Remove the ".tar.gz" suffix.  */
     nName -= 7;
@@ -594,15 +711,56 @@ void tarball_page(void){
       }
     }
   }
-  rid = name_to_rid(nRid?zRid:zName);
+  rid = name_to_typed_rid(nRid?zRid:zName, "ci");
   if( rid==0 ){
     @ Not found
     return;
   }
   if( nRid==0 && nName>10 ) zName[10] = 0;
-  tarball_of_checkin(rid, &tarball, zName);
-  free( zName );
-  free( zRid );
+
+  /* Compute a unique key for the cache entry based on query parameters */
+  blob_init(&cacheKey, 0, 0);
+  blob_appendf(&cacheKey, "/tarball/%z", rid_to_uuid(rid));
+  blob_appendf(&cacheKey, "/%q", zName);
+  if( zInclude ) blob_appendf(&cacheKey, ",in=%Q", zInclude);
+  if( zExclude ) blob_appendf(&cacheKey, ",ex=%Q", zExclude);
+  zKey = blob_str(&cacheKey);
+
+  if( P("debug")!=0 ){
+    style_header("Tarball Generator Debug Screen");
+    @ zName = "%h(zName)"<br />
+    @ rid = %d(rid)<br />
+    if( zInclude ){
+      @ zInclude = "%h(zInclude)"<br />
+    }
+    if( zExclude ){
+      @ zExclude = "%h(zExclude)"<br />
+    }
+    @ zKey = "%h(zKey)"
+    style_footer();
+    return;
+  }
+  if( referred_from_login() ){
+    style_header("Tarball Download");
+    @ <form action='%R/tarball/%h(zName).tar.gz'>
+    cgi_query_parameters_to_hidden();
+    @ <p>Tarball named <b>%h(zName).tar.gz</b> holding the content
+    @ of check-in <b>%h(zRid)</b>:
+    @ <input type="submit" value="Download" />
+    @ </form>
+    style_footer();
+    return;
+  }
+  blob_zero(&tarball);
+  if( cache_read(&tarball, zKey)==0 ){
+    tarball_of_checkin(rid, &tarball, zName, pInclude, pExclude);
+    cache_write(&tarball, zKey);
+  }
+  glob_free(pInclude);
+  glob_free(pExclude);
+  fossil_free(zName);
+  fossil_free(zRid);
+  blob_reset(&cacheKey);
   cgi_set_content(&tarball);
   cgi_set_content_type("application/x-compressed");
 }
