@@ -36,7 +36,7 @@ static int internalUpdate = 0;
 static int internalConflictCnt = 0;
 
 /*
-** Do an update to version vid.  
+** Do an update to version vid.
 **
 ** Start an undo session but do not terminate it.  Do not autosync.
 */
@@ -64,33 +64,44 @@ int update_to(int vid){
 **
 ** Usage: %fossil update ?OPTIONS? ?VERSION? ?FILES...?
 **
-** Change the version of the current checkout to VERSION.  Any uncommitted
-** changes are retained and applied to the new checkout.
+** Change the version of the current checkout to VERSION.  Any
+** uncommitted changes are retained and applied to the new checkout.
 **
-** The VERSION argument can be a specific version or tag or branch name.
-** If the VERSION argument is omitted, then the leaf of the subtree
-** that begins at the current version is used, if there is only a single
-** leaf.  VERSION can also be "current" to select the leaf of the current
-** version or "latest" to select the most recent check-in.
+** The VERSION argument can be a specific version or tag or branch
+** name.  If the VERSION argument is omitted, then the leaf of the
+** subtree that begins at the current version is used, if there is
+** only a single leaf.  VERSION can also be "current" to select the
+** leaf of the current version or "latest" to select the most recent
+** check-in.
 **
 ** If one or more FILES are listed after the VERSION then only the
-** named files are candidates to be updated.  If FILES is omitted, all
-** files in the current checkout are subject to be updated.  Using
-** a directory name for one of the FILES arguments is the same as
-** using every subdirectory and file beneath that directory.
+** named files are candidates to be updated, and any updates to them
+** will be treated as edits to the current version. Using a directory
+** name for one of the FILES arguments is the same as using every
+** subdirectory and file beneath that directory.
 **
-** The -n or --nochange option causes this command to do a "dry run".  It
-** prints out what would have happened but does not actually make any
-** changes to the current checkout or the repository.
+** If FILES is omitted, all files in the current checkout are subject
+** to being updated and the version of the current checkout is changed
+** to VERSION. Any uncommitted changes are retained and applied to the
+** new checkout.
 **
-** The -v or --verbose option prints status information about unchanged
-** files in addition to those file that actually do change.
+** The -n or --dry-run option causes this command to do a "dry run".
+** It prints out what would have happened but does not actually make
+** any changes to the current checkout or the repository.
+**
+** The -v or --verbose option prints status information about
+** unchanged files in addition to those file that actually do change.
 **
 ** Options:
+**   --case-sensitive <BOOL> override case-sensitive setting
 **   --debug          print debug information on stdout
 **   --latest         acceptable in place of VERSION, update to latest version
-**   -n|--nochange    do not perform changes but show what would be done
+**   --force-missing  force update if missing content after sync
+**   --no-dir-symlinks Disables support for directory symlinks.
+**   -n|--dry-run     If given, display instead of run actions
 **   -v|--verbose     print status information about all files
+**   -W|--width <num> Width of lines (default is to auto-detect). Must be >20
+**                    or 0 (= no limit, resulting in a single line per entry).
 **
 ** See also: revert
 */
@@ -99,8 +110,9 @@ void update_cmd(void){
   int tid=0;            /* Target version - version we are changing to */
   Stmt q;
   int latestFlag;       /* --latest.  Pick the latest version if true */
-  int nochangeFlag;     /* -n or --nochange.  Do a dry run */
+  int dryRunFlag;       /* -n or --dry-run.  Do a dry run */
   int verboseFlag;      /* -v or --verbose.  Output extra information */
+  int forceMissingFlag; /* --force-missing.  Continue if missing content */
   int debugFlag;        /* --debug option */
   int setmtimeFlag;     /* --setmtime.  Set mtimes on files */
   int nChng;            /* Number of file renames */
@@ -108,28 +120,50 @@ void update_cmd(void){
   int i;                /* Loop counter */
   int nConflict = 0;    /* Number of merge conflicts */
   int nOverwrite = 0;   /* Number of unmanaged files overwritten */
-  int nUpdate = 0;      /* Number of chagnes of any kind */
-  Stmt mtimeXfer;       /* Statment to transfer mtimes */
+  int nUpdate = 0;      /* Number of changes of any kind */
+  int width;            /* Width of printed comment lines */
+  Stmt mtimeXfer;       /* Statement to transfer mtimes */
+  const char *zWidth;   /* Width option string value */
 
   if( !internalUpdate ){
     undo_capture_command_line();
     url_proxy_options();
   }
+  zWidth = find_option("width","W",1);
+  if( zWidth ){
+    width = atoi(zWidth);
+    if( (width!=0) && (width<=20) ){
+      fossil_fatal("-W|--width value must be >20 or 0");
+    }
+  }else{
+    width = -1;
+  }
   latestFlag = find_option("latest",0, 0)!=0;
-  nochangeFlag = find_option("nochange","n",0)!=0;
+  dryRunFlag = find_option("dry-run","n",0)!=0;
+  if( !dryRunFlag ){
+    dryRunFlag = find_option("nochange",0,0)!=0; /* deprecated */
+  }
   verboseFlag = find_option("verbose","v",0)!=0;
+  forceMissingFlag = find_option("force-missing",0,0)!=0;
   debugFlag = find_option("debug",0,0)!=0;
   setmtimeFlag = find_option("setmtime",0,0)!=0;
+
+  /* We should be done with options.. */
+  verify_all_options();
+
   db_must_be_within_tree();
   vid = db_lget_int("checkout", 0);
-  if( vid==0 ){
-    fossil_fatal("cannot find current version");
+  user_select();
+  if( !dryRunFlag && !internalUpdate ){
+    if( autosync_loop(SYNC_PULL + SYNC_VERBOSE*verboseFlag,
+                      db_get_int("autosync-tries", 1), 1) ){
+      fossil_fatal("update abandoned due to sync failure");
+    }
   }
-  if( !nochangeFlag && !internalUpdate ) autosync(AUTOSYNC_PULL);
-  
+
   /* Create any empty directories now, as well as after the update,
   ** so changes in settings are reflected now */
-  if( !nochangeFlag ) ensure_empty_dirs_created();
+  if( !dryRunFlag ) ensure_empty_dirs_created();
 
   if( internalUpdate ){
     tid = internalUpdate;
@@ -144,18 +178,16 @@ void update_cmd(void){
       latestFlag = 1;
     }else{
       tid = name_to_typed_rid(g.argv[2],"ci");
-      if( tid==0 ){
-        fossil_fatal("no such version: %s", g.argv[2]);
-      }else if( !is_a_version(tid) ){
-        fossil_fatal("no such version: %s", g.argv[2]);
+       if( tid==0 || !is_a_version(tid) ){
+        fossil_fatal("no such check-in: %s", g.argv[2]);
       }
     }
   }
-  
+
   /* If no VERSION is specified on the command-line, then look for a
-  ** descendent of the current version.  If there are multiple descendents,
+  ** descendent of the current version.  If there are multiple descendants,
   ** look for one from the same branch as the current version.  If there
-  ** are still multiple descendents, show them all and refuse to update
+  ** are still multiple descendants, show them all and refuse to update
   ** until the user selects one.
   */
   if( tid==0 ){
@@ -176,31 +208,33 @@ void update_cmd(void){
       );
       if( db_int(0, "SELECT count(*) FROM leaves")>1 ){
         compute_leaves(vid, closeCode);
-        db_prepare(&q, 
+        db_prepare(&q,
           "%s "
           "   AND event.objid IN leaves"
           " ORDER BY event.mtime DESC",
           timeline_query_for_tty()
         );
-        print_timeline(&q, 100, 0);
+        print_timeline(&q, -100, width, 0);
         db_finalize(&q);
         fossil_fatal("Multiple descendants");
       }
     }
     tid = db_int(0, "SELECT rid FROM leaves, event"
                     " WHERE event.objid=leaves.rid"
-                    " ORDER BY event.mtime DESC"); 
+                    " ORDER BY event.mtime DESC");
     if( tid==0 ) tid = vid;
   }
 
   if( tid==0 ){
-    fossil_panic("Internal Error: unable to find a version to update to.");
+    return;
   }
 
   db_begin_transaction();
   vfile_check_signature(vid, CKSIG_ENOTFILE);
-  if( !nochangeFlag && !internalUpdate ) undo_begin();
-  load_vfile_from_rid(tid);
+  if( !dryRunFlag && !internalUpdate ) undo_begin();
+  if( load_vfile_from_rid(tid) && !forceMissingFlag ){
+    fossil_fatal("missing content, unable to update");
+  };
 
   /*
   ** The record.fn field is used to match files against each other.  The
@@ -210,7 +244,7 @@ void update_cmd(void){
   db_multi_exec(
     "DROP TABLE IF EXISTS fv;"
     "CREATE TEMP TABLE fv("
-    "  fn TEXT PRIMARY KEY,"      /* The filename relative to root */
+    "  fn TEXT %s PRIMARY KEY,"   /* The filename relative to root */
     "  idv INTEGER,"              /* VFILE entry for current version */
     "  idt INTEGER,"              /* VFILE entry for target version */
     "  chnged BOOLEAN,"           /* True if current version has been edited */
@@ -219,15 +253,17 @@ void update_cmd(void){
     "  ridv INTEGER,"             /* Record ID for current version */
     "  ridt INTEGER,"             /* Record ID for target */
     "  isexe BOOLEAN,"            /* Does target have execute permission? */
-    "  fnt TEXT"                  /* Filename of same file on target version */
-    ");"
+    "  deleted BOOLEAN DEFAULT 0,"/* File marked by "rm" to become unmanaged */
+    "  fnt TEXT %s"               /* Filename of same file on target version */
+    ");",
+    filename_collation(), filename_collation()
   );
 
   /* Add files found in the current version
   */
   db_multi_exec(
-    "INSERT OR IGNORE INTO fv(fn,fnt,idv,idt,ridv,ridt,isexe,chnged)"
-    " SELECT pathname, pathname, id, 0, rid, 0, isexe, chnged"
+    "INSERT OR IGNORE INTO fv(fn,fnt,idv,idt,ridv,ridt,isexe,chnged,deleted)"
+    " SELECT pathname, pathname, id, 0, rid, 0, isexe, chnged, deleted"
     "   FROM vfile WHERE vid=%d",
     vid
   );
@@ -235,17 +271,19 @@ void update_cmd(void){
   /* Compute file name changes on V->T.  Record name changes in files that
   ** have changed locally.
   */
-  find_filename_changes(vid, tid, 1, &nChng, &aChng, debugFlag ? "V->T": 0);
-  if( nChng ){
-    for(i=0; i<nChng; i++){
-      db_multi_exec(
-        "UPDATE fv"
-        "   SET fnt=(SELECT name FROM filename WHERE fnid=%d)"
-        " WHERE fn=(SELECT name FROM filename WHERE fnid=%d) AND chnged",
-        aChng[i*2+1], aChng[i*2]
-      );
+  if( vid ){
+    find_filename_changes(vid, tid, 1, &nChng, &aChng, debugFlag ? "V->T": 0);
+    if( nChng ){
+      for(i=0; i<nChng; i++){
+        db_multi_exec(
+          "UPDATE fv"
+          "   SET fnt=(SELECT name FROM filename WHERE fnid=%d)"
+          " WHERE fn=(SELECT name FROM filename WHERE fnid=%d) AND chnged",
+          aChng[i*2+1], aChng[i*2]
+        );
+      }
+      fossil_free(aChng);
     }
-    fossil_free(aChng);
   }
 
   /* Add files found in the target version T but missing from the current
@@ -255,8 +293,8 @@ void update_cmd(void){
     "INSERT OR IGNORE INTO fv(fn,fnt,idv,idt,ridv,ridt,isexe,chnged)"
     " SELECT pathname, pathname, 0, 0, 0, 0, isexe, 0 FROM vfile"
     "  WHERE vid=%d"
-    "    AND pathname NOT IN (SELECT fnt FROM fv)",
-    tid
+    "    AND pathname %s NOT IN (SELECT fnt FROM fv)",
+    tid, filename_collation()
   );
 
   /*
@@ -264,8 +302,8 @@ void update_cmd(void){
   */
   db_multi_exec(
     "UPDATE fv SET"
-    " idt=coalesce((SELECT id FROM vfile WHERE vid=%d AND pathname=fnt),0),"
-    " ridt=coalesce((SELECT rid FROM vfile WHERE vid=%d AND pathname=fnt),0)",
+    " idt=coalesce((SELECT id FROM vfile WHERE vid=%d AND fnt=pathname),0),"
+    " ridt=coalesce((SELECT rid FROM vfile WHERE vid=%d AND fnt=pathname),0)",
     tid, tid
   );
 
@@ -275,9 +313,9 @@ void update_cmd(void){
   db_multi_exec(
     "UPDATE fv SET"
     " islinkv=coalesce((SELECT islink FROM vfile"
-                       " WHERE vid=%d AND pathname=fnt),0),"
+                       " WHERE vid=%d AND fnt=pathname),0),"
     " islinkt=coalesce((SELECT islink FROM vfile"
-                       " WHERE vid=%d AND pathname=fnt),0)",
+                       " WHERE vid=%d AND fnt=pathname),0)",
     vid, tid
   );
 
@@ -317,21 +355,23 @@ void update_cmd(void){
     blob_append(&sql, "DELETE FROM fv WHERE ", -1);
     zSep = "";
     for(i=3; i<g.argc; i++){
-      file_tree_name(g.argv[i], &treename, 1);
+      file_tree_name(g.argv[i], &treename, 0, 1);
       if( file_wd_isdir(g.argv[i])==1 ){
         if( blob_size(&treename) != 1 || blob_str(&treename)[0] != '.' ){
-          blob_appendf(&sql, "%sfn NOT GLOB '%b/*' ", zSep, &treename);
+          blob_append_sql(&sql, "%sfn NOT GLOB '%q/*' ",
+                         zSep /*safe-for-%s*/, blob_str(&treename));
         }else{
           blob_reset(&sql);
           break;
         }
       }else{
-        blob_appendf(&sql, "%sfn<>%B ", zSep, &treename);
+        blob_append_sql(&sql, "%sfn<>%Q ",
+                        zSep /*safe-for-%s*/, blob_str(&treename));
       }
       zSep = "AND ";
       blob_reset(&treename);
     }
-    db_multi_exec(blob_str(&sql));
+    db_multi_exec("%s", blob_sql_text(&sql));
     blob_reset(&sql);
   }
 
@@ -339,16 +379,16 @@ void update_cmd(void){
   ** Alter the content of the checkout so that it conforms with the
   ** target
   */
-  db_prepare(&q, 
+  db_prepare(&q,
     "SELECT fn, idv, ridv, idt, ridt, chnged, fnt,"
-    "       isexe, islinkv, islinkt FROM fv ORDER BY 1"
+    "       isexe, islinkv, islinkt, deleted FROM fv ORDER BY 1"
   );
   db_prepare(&mtimeXfer,
     "UPDATE vfile SET mtime=(SELECT mtime FROM vfile WHERE id=:idv)"
     " WHERE id=:idt"
   );
   assert( g.zLocalRoot!=0 );
-  assert( strlen(g.zLocalRoot)>1 );
+  assert( strlen(g.zLocalRoot)>0 );
   assert( g.zLocalRoot[strlen(g.zLocalRoot)-1]=='/' );
   while( db_step(&q)==SQLITE_ROW ){
     const char *zName = db_column_text(&q, 0);  /* The filename from root */
@@ -361,6 +401,7 @@ void update_cmd(void){
     int isexe = db_column_int(&q, 7);           /* EXE perm for new file */
     int islinkv = db_column_int(&q, 8);         /* Is current file is a link */
     int islinkt = db_column_int(&q, 9);         /* Is target file is a link */
+    int deleted = db_column_int(&q, 10);        /* Marked for deletion */
     char *zFullPath;                            /* Full pathname of the file */
     char *zFullNewPath;                         /* Full pathname of dest */
     char nameChng;                              /* True if the name changed */
@@ -369,6 +410,9 @@ void update_cmd(void){
     zFullNewPath = mprintf("%s%s", g.zLocalRoot, zNewName);
     nameChng = fossil_strcmp(zName, zNewName);
     nUpdate++;
+    if( deleted ){
+      db_multi_exec("UPDATE vfile SET deleted=1 WHERE id=%d", idt);
+    }
     if( idv>0 && ridv==0 && idt>0 && ridt>0 ){
       /* Conflict.  This file has been added to the current checkout
       ** but also exists in the target checkout.  Use the current version.
@@ -378,24 +422,28 @@ void update_cmd(void){
     }else if( idt>0 && idv==0 ){
       /* File added in the target. */
       if( file_wd_isfile_or_link(zFullPath) ){
-        fossil_print("ADD %s (overwrites an unmanaged file)\n", zName);
+        fossil_print("ADD %s - overwrites an unmanaged file\n", zName);
         nOverwrite++;
       }else{
         fossil_print("ADD %s\n", zName);
       }
-      undo_save(zName);
-      if( !nochangeFlag ) vfile_to_disk(0, idt, 0, 0);
-    }else if( idt>0 && idv>0 && ridt!=ridv && chnged==0 ){
+      if( !dryRunFlag && !internalUpdate ) undo_save(zName);
+      if( !dryRunFlag ) vfile_to_disk(0, idt, 0, 0);
+    }else if( idt>0 && idv>0 && ridt!=ridv && (chnged==0 || deleted) ){
       /* The file is unedited.  Change it to the target version */
-      undo_save(zName);
-      fossil_print("UPDATE %s\n", zName);
-      if( !nochangeFlag ) vfile_to_disk(0, idt, 0, 0);
-    }else if( idt>0 && idv>0 && file_wd_size(zFullPath)<0 ){
+      if( deleted ){
+        fossil_print("UPDATE %s - change to unmanaged file\n", zName);
+      }else{
+        fossil_print("UPDATE %s\n", zName);
+      }
+      if( !dryRunFlag && !internalUpdate ) undo_save(zName);
+      if( !dryRunFlag ) vfile_to_disk(0, idt, 0, 0);
+    }else if( idt>0 && idv>0 && !deleted && file_wd_size(zFullPath)<0 ){
       /* The file missing from the local check-out. Restore it to the
       ** version that appears in the target. */
       fossil_print("UPDATE %s\n", zName);
-      undo_save(zName);
-      if( !nochangeFlag ) vfile_to_disk(0, idt, 0, 0);
+      if( !dryRunFlag && !internalUpdate ) undo_save(zName);
+      if( !dryRunFlag ) vfile_to_disk(0, idt, 0, 0);
     }else if( idt==0 && idv>0 ){
       if( ridv==0 ){
         /* Added in current checkout.  Continue to hold the file as
@@ -409,8 +457,8 @@ void update_cmd(void){
         nConflict++;
       }else{
         fossil_print("REMOVE %s\n", zName);
-        undo_save(zName);
-        if( !nochangeFlag ) file_delete(zFullPath);
+        if( !dryRunFlag && !internalUpdate ) undo_save(zName);
+        if( !dryRunFlag ) file_delete(zFullPath);
       }
     }else if( idt>0 && idv>0 && ridt!=ridv && chnged ){
       /* Merge the changes in the current tree into the target version */
@@ -423,14 +471,15 @@ void update_cmd(void){
       }
       if( islinkv || islinkt /* || file_wd_islink(zFullPath) */ ){
         fossil_print("***** Cannot merge symlink %s\n", zNewName);
-        nConflict++;        
+        nConflict++;
       }else{
-        undo_save(zName);
+        unsigned mergeFlags = dryRunFlag ? MERGE_DRYRUN : 0;
+        if( !dryRunFlag && !internalUpdate ) undo_save(zName);
         content_get(ridt, &t);
         content_get(ridv, &v);
-        rc = merge_3way(&v, zFullPath, &t, &r);
+        rc = merge_3way(&v, zFullPath, &t, &r, mergeFlags);
         if( rc>=0 ){
-          if( !nochangeFlag ){
+          if( !dryRunFlag ){
             blob_write_to_file(&r, zFullNewPath);
             file_wd_setexe(zFullNewPath, isexe);
           }
@@ -439,7 +488,7 @@ void update_cmd(void){
             nConflict++;
           }
         }else{
-          if( !nochangeFlag ){
+          if( !dryRunFlag ){
             blob_write_to_file(&t, zFullNewPath);
             file_wd_setexe(zFullNewPath, isexe);
           }
@@ -447,7 +496,7 @@ void update_cmd(void){
           nConflict++;
         }
       }
-      if( nameChng && !nochangeFlag ) file_delete(zFullPath);
+      if( nameChng && !dryRunFlag ) file_delete(zFullPath);
       blob_reset(&v);
       blob_reset(&t);
       blob_reset(&r);
@@ -480,7 +529,7 @@ void update_cmd(void){
 
   /* Report on conflicts
   */
-  if( !nochangeFlag ){
+  if( !dryRunFlag ){
     Stmt q;
     int nMerge = 0;
     db_prepare(&q, "SELECT uuid, id FROM vmerge JOIN blob ON merge=rid"
@@ -496,7 +545,8 @@ void update_cmd(void){
       nMerge++;
     }
     db_finalize(&q);
-    
+    leaf_ambiguity_warning(tid, tid);
+
     if( nConflict ){
       if( internalUpdate ){
         internalConflictCnt = nConflict;
@@ -513,12 +563,12 @@ void update_cmd(void){
       fossil_warning("WARNING: %d uncommitted prior merges", nMerge);
     }
   }
-  
+
   /*
   ** Clean up the mid and pid VFILE entries.  Then commit the changes.
   */
-  if( nochangeFlag ){
-    db_end_transaction(1);  /* With --nochange, rollback changes */
+  if( dryRunFlag ){
+    db_end_transaction(1);  /* With --dry-run, rollback changes */
   }else{
     ensure_empty_dirs_created();
     if( g.argc<=3 ){
@@ -533,48 +583,37 @@ void update_cmd(void){
       db_multi_exec("DELETE FROM vfile WHERE vid!=%d", vid);
     }
     if( !internalUpdate ) undo_finish();
-    if( setmtimeFlag ) vfile_check_signature(vid, CKSIG_SETMTIME);
+    if( setmtimeFlag ) vfile_check_signature(tid, CKSIG_SETMTIME);
     db_end_transaction(0);
   }
 }
 
 /*
-** Make sure empty directories are created
+** Create empty directories specified by the empty-dirs setting.
 */
 void ensure_empty_dirs_created(void){
-  /* Make empty directories? */
   char *zEmptyDirs = db_get("empty-dirs", 0);
   if( zEmptyDirs!=0 ){
-    char *bc;
+    int i;
     Blob dirName;
     Blob dirsList;
 
-    blob_zero(&dirsList);
-    blob_init(&dirsList, zEmptyDirs, strlen(zEmptyDirs));
-    /* Replace commas by spaces */
-    bc = blob_str(&dirsList);
-    while( (*bc)!='\0' ){
-      if( (*bc)==',' ) { *bc = ' '; }
-      ++bc;
+    zEmptyDirs = fossil_strdup(zEmptyDirs);
+    for(i=0; zEmptyDirs[i]; i++){
+      if( zEmptyDirs[i]==',' ) zEmptyDirs[i] = ' ';
     }
-    /* Make directories */
-    blob_zero(&dirName);
+    blob_init(&dirsList, zEmptyDirs, -1);
     while( blob_token(&dirsList, &dirName) ){
-      const char *zDir = blob_str(&dirName);
-      /* Make full pathname of the directory */
-      Blob path;
-      const char *zPath;
-
-      blob_zero(&path);
-      blob_appendf(&path, "%s/%s", g.zLocalRoot, zDir);
-      zPath = blob_str(&path);      
-      /* Handle various cases of existence of the directory */
+      char *zDir = blob_str(&dirName);
+      char *zPath = mprintf("%s/%s", g.zLocalRoot, zDir);
       switch( file_wd_isdir(zPath) ){
         case 0: { /* doesn't exist */
-          if( file_mkdir(zPath, 0)!=0 ) {
+          fossil_free(zPath);
+          zPath = mprintf("%s/%s/x", g.zLocalRoot, zDir);
+          if( file_mkfolder(zPath, 0, 1)!=0 ) {
             fossil_warning("couldn't create directory %s as "
                            "required by empty-dirs setting", zDir);
-          }          
+          }
           break;
         }
         case 1: { /* exists, and is a directory */
@@ -583,70 +622,107 @@ void ensure_empty_dirs_created(void){
         }
         case 2: { /* exists, but isn't a directory */
           fossil_warning("file %s found, but a directory is required "
-                         "by empty-dirs setting", zDir);          
+                         "by empty-dirs setting", zDir);
         }
       }
-      blob_reset(&path);
+      fossil_free(zPath);
+      blob_reset(&dirName);
     }
+    blob_reset(&dirsList);
+    fossil_free(zEmptyDirs);
   }
 }
 
-
 /*
-** Get the contents of a file within the checking "revision".  If
-** revision==NULL then get the file content for the current checkout.
+** Get the manifest record for a given revision, or the current checkout if
+** zRevision is NULL.
 */
-int historical_version_of_file(
-  const char *revision,    /* The checkin containing the file */
-  const char *file,        /* Full treename of the file */
-  Blob *content,           /* Put the content here */
-  int *pIsLink,            /* Set to true if file is link. */
-  int *pIsExe,             /* Set to true if file is executable */
-  int *pIsBin,             /* Set to true if file is binary */
-  int errCode              /* Error code if file not found.  Panic if 0. */
+Manifest *historical_manifest(
+  const char *zRevision    /* The check-in to query, or NULL for current */
 ){
+  int vid;
   Manifest *pManifest;
-  ManifestFile *pFile;
-  int rid=0;
-  
-  if( revision ){
-    rid = name_to_typed_rid(revision,"ci");
+
+  /* Determine the check-in manifest artifact ID.  Panic on failure. */
+  if( zRevision ){
+    vid = name_to_typed_rid(zRevision, "ci");
+  }else if( !g.localOpen ){
+    vid = name_to_typed_rid(db_get("main-branch", "trunk"), "ci");
   }else{
-    rid = db_lget_int("checkout", 0);
-  }
-  if( !is_a_version(rid) ){
-    if( errCode>0 ) return errCode;
-    fossil_fatal("no such checkin: %s", revision);
-  }
-  pManifest = manifest_get(rid, CFTYPE_MANIFEST);
-  
-  if( pManifest ){
-    pFile = manifest_file_find(pManifest, file);
-    if( pFile ){
-      int rc;
-      rid = uuid_to_rid(pFile->zUuid, 0);
-      if( pIsExe ) *pIsExe = ( manifest_file_mperm(pFile)==PERM_EXE );
-      if( pIsLink ) *pIsLink = ( manifest_file_mperm(pFile)==PERM_LNK );
-      manifest_destroy(pManifest);
-      rc = content_get(rid, content);
-      if( rc && pIsBin ){
-        *pIsBin = looks_like_binary(content);
+    vid = db_lget_int("checkout", 0);
+    if( !is_a_version(vid) ){
+      zRevision = db_text(0, "SELECT uuid FROM blob WHERE rid=%d", vid);
+      if( zRevision ){
+        fossil_fatal("checkout artifact is not a check-in: %s", zRevision);
+      }else{
+        fossil_fatal("invalid checkout artifact ID: %d", vid);
       }
-      return rc;
     }
-    manifest_destroy(pManifest);
-    if( errCode<=0 ){
-      fossil_fatal("file %s does not exist in checkin: %s", file, revision);
-    }
-  }else if( errCode<=0 ){
-    if( revision==0 ){
-      revision = db_text("current", "SELECT uuid FROM blob WHERE rid=%d", rid);
-    }
-    fossil_panic("could not parse manifest for checkin: %s", revision);
   }
-  return errCode;
+
+  /* Parse the manifest, given its artifact ID.  Panic on failure. */
+  if( !(pManifest = manifest_get(vid, CFTYPE_MANIFEST, 0)) ){
+    if( zRevision ){
+      fossil_fatal("could not parse manifest for check-in: %s", zRevision);
+    }else{
+      fossil_fatal("could not parse manifest for current checkout");
+    }
+  }
+
+  /* Return the manifest pointer.  The caller must use manifest_destroy() to
+   * clean up when finished using the manifest. */
+  return pManifest;
 }
 
+/*
+** Get the contents of a file within the check-in "zRevision".  If
+** zRevision==NULL then get the file content for the current checkout.
+*/
+int historical_blob(
+  const char *zRevision,   /* The check-in containing the file */
+  const char *zFile,       /* Full treename of the file */
+  Blob *pBlob,             /* Put the content here */
+  int fatal                /* If nonzero, panic if file/artifact not found */
+){
+  int result = 0;
+
+  /* Get the manifest for the requested check-in version.  This call unavoidably
+   * panics on failure even if fatal is not set. */
+  Manifest *pManifest = historical_manifest(zRevision);
+
+  /* Try to find the file record within the manifest. */
+  ManifestFile *pFile = manifest_file_find(pManifest, zFile);
+
+  if( !pFile ){
+    /* Process file-not-found errors. */
+    if( fatal ){
+      if( zRevision ){
+        fossil_fatal("file %s does not exist in check-in %s", zFile, zRevision);
+      }else{
+        fossil_fatal("no such file: %s", zFile);
+      }
+    }
+  }else{
+    /* Get the file's contents. */
+    result = content_get(fast_uuid_to_rid(pFile->zUuid), pBlob);
+
+    /* Process artifact-not-found errors. */
+    if( !result && fatal ){
+      if( zRevision ){
+        fossil_fatal("missing artifact %s for file %s in check-in %s",
+            pFile->zUuid, zFile, zRevision);
+      }else{
+        fossil_fatal("missing artifact %s for file %s", pFile->zUuid, zFile);
+      }
+    }
+  }
+
+  /* Deallocate the parsed manifest structure. */
+  manifest_destroy(pManifest);
+
+  /* Return 1 on success and (assuming fatal is not set) 0 if not found. */
+  return result;
+}
 
 /*
 ** COMMAND: revert
@@ -657,6 +733,9 @@ int historical_version_of_file(
 ** the version associated with baseline REVISION if the -r flag
 ** appears.
 **
+** If FILE was part of a rename operation, both the original file
+** and the renamed file are reverted.
+**
 ** Revert all files if no file name is provided.
 **
 ** If a file is reverted accidently, it can be restored using
@@ -668,17 +747,20 @@ int historical_version_of_file(
 ** See also: redo, undo, update
 */
 void revert_cmd(void){
-  const char *zFile;
-  const char *zRevision;
-  Blob record;
+  Manifest *pCoManifest;          /* Manifest of current checkout */
+  Manifest *pRvManifest;          /* Manifest of selected revert version */
+  ManifestFile *pCoFile;          /* File within current checkout manifest */
+  ManifestFile *pRvFile;          /* File within revert version manifest */
+  const char *zFile;              /* Filename relative to checkout root */
+  const char *zRevision;          /* Selected revert version, NULL if current */
+  Blob record = BLOB_INITIALIZER; /* Contents of each reverted file */
   int i;
-  int errCode;
   Stmt q;
 
-  undo_capture_command_line();  
+  undo_capture_command_line();
   zRevision = find_option("revision", "r", 1);
   verify_all_options();
-  
+
   if( g.argc<2 ){
     usage("?OPTIONS? [FILE] ...");
   }
@@ -686,6 +768,11 @@ void revert_cmd(void){
     fossil_fatal("the --revision option does not work for the entire tree");
   }
   db_must_be_within_tree();
+
+  /* Get manifests of revert version and (if different) current checkout. */
+  pRvManifest = historical_manifest(zRevision);
+  pCoManifest = zRevision ? historical_manifest(0) : 0;
+
   db_begin_transaction();
   undo_begin();
   db_multi_exec("CREATE TEMP TABLE torevert(name UNIQUE);");
@@ -694,8 +781,16 @@ void revert_cmd(void){
     for(i=2; i<g.argc; i++){
       Blob fname;
       zFile = mprintf("%/", g.argv[i]);
-      file_tree_name(zFile, &fname, 1);
-      db_multi_exec("REPLACE INTO torevert VALUES(%B)", &fname);
+      blob_zero(&fname);
+      file_tree_name(zFile, &fname, 0, 1);
+      db_multi_exec(
+        "REPLACE INTO torevert VALUES(%B);"
+        "INSERT OR IGNORE INTO torevert"
+        " SELECT pathname"
+        "   FROM vfile"
+        "  WHERE origname=%B;",
+        &fname, &fname
+      );
       blob_reset(&fname);
     }
   }else{
@@ -704,12 +799,18 @@ void revert_cmd(void){
     vfile_check_signature(vid, 0);
     db_multi_exec(
       "DELETE FROM vmerge;"
-      "INSERT INTO torevert "
-      "SELECT pathname"
-      "  FROM vfile "
-      " WHERE chnged OR deleted OR rid=0 OR pathname!=origname;"
+      "INSERT OR IGNORE INTO torevert "
+      " SELECT pathname"
+      "   FROM vfile "
+      "  WHERE chnged OR deleted OR rid=0 OR pathname!=origname;"
     );
   }
+  db_multi_exec(
+    "INSERT OR IGNORE INTO torevert"
+    " SELECT origname"
+    "   FROM vfile"
+    "  WHERE origname!=pathname AND pathname IN (SELECT name FROM torevert);"
+  );
   blob_zero(&record);
   db_prepare(&q, "SELECT name FROM torevert");
   if( zRevision==0 ){
@@ -717,42 +818,57 @@ void revert_cmd(void){
     zRevision = db_text(0, "SELECT uuid FROM blob WHERE rid=%d", vid);
   }
   while( db_step(&q)==SQLITE_ROW ){
-    int isExe = 0;
-    int isLink = 0;
     char *zFull;
     zFile = db_column_text(&q, 0);
     zFull = mprintf("%/%/", g.zLocalRoot, zFile);
-    errCode = historical_version_of_file(zRevision, zFile, &record,
-                                         &isLink, &isExe, 0, 2);
-    if( errCode==2 ){
-      if( db_int(0, "SELECT rid FROM vfile WHERE pathname=%Q", zFile)==0 ){
-        fossil_print("UNMANAGE: %s\n", zFile);
+    pRvFile = manifest_file_find(pRvManifest, zFile);
+    if( !pRvFile ){
+      if( db_int(0, "SELECT rid FROM vfile WHERE pathname=%Q OR origname=%Q",
+                 zFile, zFile)==0 ){
+        fossil_print("UNMANAGE %s\n", zFile);
       }else{
         undo_save(zFile);
         file_delete(zFull);
-        fossil_print("DELETE: %s\n", zFile);
+        fossil_print("DELETE   %s\n", zFile);
       }
-      db_multi_exec("DELETE FROM vfile WHERE pathname=%Q", zFile);
+      db_multi_exec(
+        "UPDATE OR REPLACE vfile"
+        "   SET pathname=origname, origname=NULL"
+        " WHERE pathname=%Q AND origname!=pathname;"
+        "DELETE FROM vfile WHERE pathname=%Q",
+        zFile, zFile
+      );
     }else{
       sqlite3_int64 mtime;
+      int rvChnged = 0;
+      int rvPerm = manifest_file_mperm(pRvFile);
+
+      /* Determine if reverted-to file is different than checked out file. */
+      if( pCoManifest && (pCoFile = manifest_file_find(pCoManifest, zFile)) ){
+        rvChnged = manifest_file_mperm(pRvFile)!=rvPerm
+                || fossil_strcmp(pRvFile->zUuid, pCoFile->zUuid)!=0;
+      }
+
+      /* Get contents of reverted-to file. */
+      content_get(fast_uuid_to_rid(pRvFile->zUuid), &record);
+
       undo_save(zFile);
-      if( file_wd_size(zFull)>=0 && (isLink || file_wd_islink(zFull)) ){
+      if( file_wd_size(zFull)>=0 && (rvPerm==PERM_LNK || file_wd_islink(0)) ){
         file_delete(zFull);
       }
-      if( isLink ){
+      if( rvPerm==PERM_LNK ){
         symlink_create(blob_str(&record), zFull);
       }else{
         blob_write_to_file(&record, zFull);
       }
-      file_wd_setexe(zFull, isExe);
-      fossil_print("REVERTED: %s\n", zFile);
+      file_wd_setexe(zFull, rvPerm==PERM_EXE);
+      fossil_print("REVERT   %s\n", zFile);
       mtime = file_wd_mtime(zFull);
       db_multi_exec(
          "UPDATE vfile"
-         "   SET mtime=%lld, chnged=0, deleted=0, isexe=%d, islink=%d,mrid=rid,"
-         "       pathname=coalesce(origname,pathname), origname=NULL"     
-         " WHERE pathname=%Q",
-         mtime, isExe, isLink, zFile
+         "   SET mtime=%lld, chnged=%d, deleted=0, isexe=%d, islink=%d,mrid=rid"
+         " WHERE pathname=%Q OR origname=%Q",
+         mtime, rvChnged, rvPerm==PERM_EXE, rvPerm==PERM_LNK, zFile, zFile
       );
     }
     blob_reset(&record);
@@ -761,4 +877,8 @@ void revert_cmd(void){
   db_finalize(&q);
   undo_finish();
   db_end_transaction(0);
+
+  /* Deallocate parsed manifest structures. */
+  manifest_destroy(pRvManifest);
+  manifest_destroy(pCoManifest);
 }
