@@ -23,8 +23,8 @@
 #include <sys/types.h>
 
 /*
-** The input is guaranteed to be a 40-character well-formed UUID.
-** Find its rid.
+** The input is guaranteed to be a 40- or 64-character well-formed
+** artifact hash.  Find its rid.
 */
 int fast_uuid_to_rid(const char *zUuid){
   static Stmt q;
@@ -53,13 +53,13 @@ int fast_uuid_to_rid(const char *zUuid){
 */
 int uuid_to_rid(const char *zUuid, int phantomize){
   int rid, sz;
-  char z[UUID_SIZE+1];
+  char z[HNAME_MAX+1];
 
   sz = strlen(zUuid);
-  if( sz!=UUID_SIZE || !validate16(zUuid, sz) ){
-    return 0;
+  if( !hname_validate(zUuid, sz) ){
+    return 0;  /* Not a valid hash */
   }
-  memcpy(z, zUuid, UUID_SIZE+1);
+  memcpy(z, zUuid, sz+1);
   canonical16(z, sz);
   rid = fast_uuid_to_rid(z);
   if( rid==0 && phantomize ){
@@ -132,7 +132,7 @@ int load_vfile_from_rid(int vid){
 ** combination of the following bits:
 */
 #define CKSIG_ENOTFILE  0x001   /* non-file FS objects throw an error */
-#define CKSIG_SHA1      0x002   /* Verify file content using sha1sum */
+#define CKSIG_HASH      0x002   /* Verify file content using hashing */
 #define CKSIG_SETMTIME  0x004   /* Set mtime to last check-out time */
 
 #endif /* INTERFACE */
@@ -161,8 +161,8 @@ int load_vfile_from_rid(int vid){
 ** changed without having to look at the mtime or on-disk content.
 **
 ** The mtime of the file is only a factor if the mtime-changes setting
-** is false and the useSha1sum flag is false.  If the mtime-changes
-** setting is true (or undefined - it defaults to true) or if useSha1sum
+** is false and the CKSIG_HASH flag is false.  If the mtime-changes
+** setting is true (or undefined - it defaults to true) or if CKSIG_HASH
 ** is true, then we do not trust the mtime and will examine the on-disk
 ** content to determine if a file really is the same.
 **
@@ -173,8 +173,7 @@ int load_vfile_from_rid(int vid){
 void vfile_check_signature(int vid, unsigned int cksigFlags){
   int nErr = 0;
   Stmt q;
-  Blob fileCksum, origCksum;
-  int useMtime = (cksigFlags & CKSIG_SHA1)==0
+  int useMtime = (cksigFlags & CKSIG_HASH)==0
                     && db_get_boolean("mtime-changes", 1);
 
   db_begin_transaction();
@@ -224,36 +223,26 @@ void vfile_check_signature(int vid, unsigned int cksigFlags){
     if( origSize!=currentSize ){
       if( chnged!=1 ){
         /* A file size change is definitive - the file has changed.  No
-        ** need to check the mtime or sha1sum */
+        ** need to check the mtime or hash */
         chnged = 1;
       }
     }else if( chnged==1 && rid!=0 && !isDeleted ){
       /* File is believed to have changed but it is the same size.
       ** Double check that it really has changed by looking at content. */
+      const char *zUuid = db_column_text(&q, 5);
+      int nUuid = db_column_bytes(&q, 5);
       assert( origSize==currentSize );
-      db_ephemeral_blob(&q, 5, &origCksum);
-      if( sha1sum_file(zName, &fileCksum) ){
-        blob_zero(&fileCksum);
-      }
-      if( blob_compare(&fileCksum, &origCksum)==0 ) chnged = 0;
-      blob_reset(&origCksum);
-      blob_reset(&fileCksum);
+      if( hname_verify_file_hash(zName, zUuid, nUuid) ) chnged = 0;
     }else if( (chnged==0 || chnged==2 || chnged==4)
            && (useMtime==0 || currentMtime!=oldMtime) ){
       /* For files that were formerly believed to be unchanged or that were
       ** changed by merging, if their mtime changes, or unconditionally
-      ** if --sha1sum is used, check to see if they have been edited by
-      ** looking at their SHA1 sum */
+      ** if --hash is used, check to see if they have been edited by
+      ** looking at their artifact hashes */
+      const char *zUuid = db_column_text(&q, 5);
+      int nUuid = db_column_bytes(&q, 5);
       assert( origSize==currentSize );
-      db_ephemeral_blob(&q, 5, &origCksum);
-      if( sha1sum_file(zName, &fileCksum) ){
-        blob_zero(&fileCksum);
-      }
-      if( blob_compare(&fileCksum, &origCksum) ){
-        chnged = 1;
-      }
-      blob_reset(&origCksum);
-      blob_reset(&fileCksum);
+      if( !hname_verify_file_hash(zName, zUuid, nUuid) ) chnged = 1;
     }
     if( (cksigFlags & CKSIG_SETMTIME) && (chnged==0 || chnged==2 || chnged==4) ){
       i64 desiredMtime;
@@ -265,13 +254,15 @@ void vfile_check_signature(int vid, unsigned int cksigFlags){
       }
     }
 #ifndef _WIN32
-    if( chnged==0 || chnged==6 || chnged==7 || chnged==8 || chnged==9 ){
+    if( origPerm!=PERM_LNK && currentPerm==PERM_LNK ){
+       /* Changing to a symlink takes priority over all other change types. */
+       chnged = 7;
+    }else if( chnged==0 || chnged==6 || chnged==7 || chnged==8 || chnged==9 ){
+       /* Confirm metadata change types. */
       if( origPerm==currentPerm ){
         chnged = 0;
       }else if( currentPerm==PERM_EXE ){
         chnged = 6;
-      }else if( currentPerm==PERM_LNK ){
-        chnged = 7;
       }else if( origPerm==PERM_EXE ){
         chnged = 8;
       }else if( origPerm==PERM_LNK ){
@@ -437,7 +428,7 @@ static int is_temporary_file(const char *zName){
       return 1;
     }
     if( zName[0]!='-' ) continue;
-    for(i=0; i<sizeof(azTemp)/sizeof(azTemp[0]); i++){
+    for(i=0; i<count(azTemp); i++){
       n = (int)strlen(azTemp[i]);
       if( memcmp(azTemp[i], zName+1, n) ) continue;
       if( zName[n+1]==0 ) return 1;
@@ -457,6 +448,8 @@ static int is_temporary_file(const char *zName){
 #define SCAN_ALL    0x001    /* Includes files that begin with "." */
 #define SCAN_TEMP   0x002    /* Only Fossil-generated files like *-baseline */
 #define SCAN_NESTED 0x004    /* Scan for empty dirs in nested checkouts */
+#define SCAN_MTIME  0x008    /* Populate mtime column */
+#define SCAN_SIZE   0x010    /* Populate size column */
 #endif /* INTERFACE */
 
 /*
@@ -500,9 +493,14 @@ void vfile_scan(
 
   if( depth==0 ){
     db_prepare(&ins,
-       "INSERT OR IGNORE INTO sfile(x) SELECT :file"
-       "  WHERE NOT EXISTS(SELECT 1 FROM vfile WHERE"
-       " pathname=:file %s)", filename_collation()
+      "INSERT OR IGNORE INTO sfile(pathname%s%s) SELECT :file%s%s"
+      "  WHERE NOT EXISTS(SELECT 1 FROM vfile WHERE"
+      " pathname=:file %s)",
+      scanFlags & SCAN_MTIME ? ", mtime"  : "",
+      scanFlags & SCAN_SIZE  ? ", size"   : "",
+      scanFlags & SCAN_MTIME ? ", :mtime" : "",
+      scanFlags & SCAN_SIZE  ? ", :size"  : "",
+      filename_collation()
     );
   }
   depth++;
@@ -541,6 +539,12 @@ void vfile_scan(
 #endif
         if( (scanFlags & SCAN_TEMP)==0 || is_temporary_file(zUtf8) ){
           db_bind_text(&ins, ":file", &zPath[nPrefix+1]);
+          if( scanFlags & SCAN_MTIME ){
+            db_bind_int(&ins, ":mtime", file_mtime(zPath));
+          }
+          if( scanFlags & SCAN_SIZE ){
+            db_bind_int(&ins, ":size", file_size(zPath));
+          }
           db_step(&ins);
           db_reset(&ins);
         }
