@@ -17,9 +17,14 @@
 **
 ** This file contains code used to generate ZIP archives.
 */
-#include <assert.h>
-#include <zlib.h>
 #include "config.h"
+#include <assert.h>
+#if defined(FOSSIL_ENABLE_MINIZ)
+#  define MINIZ_HEADER_FILE_ONLY
+#  include "miniz.c"
+#else
+#  include <zlib.h>
+#endif
 #include "zip.h"
 
 /*
@@ -83,7 +88,7 @@ void zip_set_timedate_from_str(const char *zDate){
 void zip_set_timedate(double rDate){
   char *zDate = db_text(0, "SELECT datetime(%.17g)", rDate);
   zip_set_timedate_from_str(zDate);
-  free(zDate);
+  fossil_free(zDate);
   unixTime = (rDate - 2440587.5)*86400.0;
 }
 
@@ -139,14 +144,14 @@ void zip_add_file(const char *zName, const Blob *pFile, int mPerm){
   /* Fill in as much of the header as we know.
   */
   nBlob = pFile ? blob_size(pFile) : 0;
-  if( nBlob>0 ){
-    iMethod = 8;
+  if( pFile ){ /* This is a file, possibly empty... */
+    iMethod = (nBlob>0) ? 8 : 0; /* Cannot compress zero bytes. */
     switch( mPerm ){
       case PERM_LNK:   iMode = 0120755;   break;
       case PERM_EXE:   iMode = 0100755;   break;
       default:         iMode = 0100644;   break;
     }
-  }else{
+  }else{       /* This is a directory, no blob... */
     iMethod = 0;
     iMode = 040755;
   }
@@ -154,19 +159,19 @@ void zip_add_file(const char *zName, const Blob *pFile, int mPerm){
   memset(zHdr, 0, sizeof(zHdr));
   put32(&zHdr[0], 0x04034b50);
   put16(&zHdr[4], 0x000a);
-  put16(&zHdr[6], 0);
+  put16(&zHdr[6], 0x0800);
   put16(&zHdr[8], iMethod);
   put16(&zHdr[10], dosTime);
   put16(&zHdr[12], dosDate);
   put16(&zHdr[26], nameLen);
   put16(&zHdr[28], 13);
-  
+
   put16(&zExTime[0], 0x5455);
   put16(&zExTime[2], 9);
   zExTime[4] = 3;
   put32(&zExTime[5], unixTime);
   put32(&zExTime[9], unixTime);
-  
+
 
   /* Write the header and filename.
   */
@@ -204,7 +209,7 @@ void zip_add_file(const char *zName, const Blob *pFile, int mPerm){
     nByte = stream.total_in;
     nByteCompr = stream.total_out;
     deflateEnd(&stream);
-  
+
     /* Go back and write the header, now that we know the compressed file size.
     */
     z = &blob_buffer(&body)[iStart];
@@ -212,14 +217,14 @@ void zip_add_file(const char *zName, const Blob *pFile, int mPerm){
     put32(&z[18], nByteCompr);
     put32(&z[22], nByte);
   }
-  
+
   /* Make an entry in the tables of contents
   */
   memset(zBuf, 0, sizeof(zBuf));
   put32(&zBuf[0], 0x02014b50);
   put16(&zBuf[4], 0x0317);
   put16(&zBuf[6], 0x000a);
-  put16(&zBuf[8], 0);
+  put16(&zBuf[8], 0x0800);
   put16(&zBuf[10], iMethod);
   put16(&zBuf[12], dosTime);
   put16(&zBuf[14], dosDate);
@@ -269,9 +274,9 @@ void zip_close(Blob *pZip){
   blob_zero(&body);
   nEntry = 0;
   for(i=0; i<nDir; i++){
-    free(azDir[i]);
+    fossil_free(azDir[i]);
   }
-  free(azDir);
+  fossil_free(azDir);
   nDir = 0;
   azDir = 0;
 }
@@ -306,7 +311,7 @@ void filezip_cmd(void){
 **
 ** If RID is for an object that is not a real manifest, then the
 ** resulting ZIP archive contains a single file which is the RID
-** object.
+** object.  The pInclude and pExclude parameters are ignored in this case.
 **
 ** If the RID object does not exist in the repository, then
 ** pZip is zeroed.
@@ -318,19 +323,25 @@ void filezip_cmd(void){
 ** with source files. For example, pass a UUID or "ProjectName".
 **
 */
-void zip_of_baseline(int rid, Blob *pZip, const char *zDir){
+void zip_of_checkin(
+  int rid,            /* The RID of the checkin to construct the ZIP archive from */
+  Blob *pZip,         /* Write the ZIP archive content into this blob */
+  const char *zDir,   /* Top-level directory of the ZIP archive */
+  Glob *pInclude,     /* Only include files that match this pattern */
+  Glob *pExclude      /* Exclude files that match this pattern */
+){
   Blob mfile, hash, file;
   Manifest *pManifest;
   ManifestFile *pFile;
   Blob filename;
   int nPrefix;
-  
+
   content_get(rid, &mfile);
   if( blob_size(&mfile)==0 ){
     blob_zero(pZip);
     return;
   }
-  blob_zero(&hash);
+  blob_set_dynamic(&hash, rid_to_uuid(rid));
   blob_zero(&filename);
   zip_open();
 
@@ -339,27 +350,68 @@ void zip_of_baseline(int rid, Blob *pZip, const char *zDir){
   }
   nPrefix = blob_size(&filename);
 
-  pManifest = manifest_get(rid, CFTYPE_MANIFEST);
+  pManifest = manifest_get(rid, CFTYPE_MANIFEST, 0);
   if( pManifest ){
-    char *zName;
+    int flg, eflg = 0;
+    char *zName = 0;
     zip_set_timedate(pManifest->rDate);
-    if( db_get_boolean("manifest", 0) ){
-      blob_append(&filename, "manifest", -1);
-      zName = blob_str(&filename);
-      zip_add_folders(zName);
-      zip_add_file(zName, &mfile, 0);
-      sha1sum_blob(&mfile, &hash);
+    flg = db_get_manifest_setting();
+    if( flg ){
+      /* eflg is the effective flags, taking include/exclude into account */
+      if( (pInclude==0 || glob_match(pInclude, "manifest"))
+       && !glob_match(pExclude, "manifest")
+       && (flg & MFESTFLG_RAW) ){
+        eflg |= MFESTFLG_RAW;
+      }
+      if( (pInclude==0 || glob_match(pInclude, "manifest.uuid"))
+       && !glob_match(pExclude, "manifest.uuid")
+       && (flg & MFESTFLG_UUID) ){
+        eflg |= MFESTFLG_UUID;
+      }
+      if( (pInclude==0 || glob_match(pInclude, "manifest.tags"))
+       && !glob_match(pExclude, "manifest.tags")
+       && (flg & MFESTFLG_TAGS) ){
+        eflg |= MFESTFLG_TAGS;
+      }
+
+      if( eflg & (MFESTFLG_RAW|MFESTFLG_UUID) ){
+        if( eflg & MFESTFLG_RAW ){
+          blob_append(&filename, "manifest", -1);
+          zName = blob_str(&filename);
+          zip_add_folders(zName);
+        }
+        if( eflg & MFESTFLG_RAW ){
+          sterilize_manifest(&mfile);
+          zip_add_file(zName, &mfile, 0);
+        }
+      }
       blob_reset(&mfile);
-      blob_append(&hash, "\n", 1);
-      blob_resize(&filename, nPrefix);
-      blob_append(&filename, "manifest.uuid", -1);
-      zName = blob_str(&filename);
-      zip_add_file(zName, &hash, 0);
-      blob_reset(&hash);
+      if( eflg & MFESTFLG_UUID ){
+        blob_append(&hash, "\n", 1);
+        blob_resize(&filename, nPrefix);
+        blob_append(&filename, "manifest.uuid", -1);
+        zName = blob_str(&filename);
+        zip_add_folders(zName);
+        zip_add_file(zName, &hash, 0);
+      }
+      if( eflg & MFESTFLG_TAGS ){
+        Blob tagslist;
+        blob_zero(&tagslist);
+        get_checkin_taglist(rid, &tagslist);
+        blob_resize(&filename, nPrefix);
+        blob_append(&filename, "manifest.tags", -1);
+        zName = blob_str(&filename);
+        zip_add_folders(zName);
+        zip_add_file(zName, &tagslist, 0);
+        blob_reset(&tagslist);
+      }
     }
     manifest_file_rewind(pManifest);
     while( (pFile = manifest_file_next(pManifest,0))!=0 ){
-      int fid = uuid_to_rid(pFile->zUuid, 0);
+      int fid;
+      if( pInclude!=0 && !glob_match(pInclude, pFile->zName) ) continue;
+      if( glob_match(pExclude, pFile->zName) ) continue;
+      fid = uuid_to_rid(pFile->zUuid, 0);
       if( fid ){
         content_get(fid, &file);
         blob_resize(&filename, nPrefix);
@@ -375,30 +427,59 @@ void zip_of_baseline(int rid, Blob *pZip, const char *zDir){
   }
   manifest_destroy(pManifest);
   blob_reset(&filename);
+  blob_reset(&hash);
   zip_close(pZip);
 }
 
 /*
 ** COMMAND: zip*
 **
-** Usage: %fossil zip VERSION OUTPUTFILE [--name DIRECTORYNAME] [-R|--repository REPO]
+** Usage: %fossil zip VERSION OUTPUTFILE [OPTIONS]
 **
-** Generate a ZIP archive for a specified version.  If the --name option is
-** used, it argument becomes the name of the top-level directory in the
+** Generate a ZIP archive for a check-in.  If the --name option is
+** used, its argument becomes the name of the top-level directory in the
 ** resulting ZIP archive.  If --name is omitted, the top-level directory
-** named is derived from the project name, the check-in date and time, and
+** name is derived from the project name, the check-in date and time, and
 ** the artifact ID of the check-in.
+**
+** The GLOBLIST argument to --exclude and --include can be a comma-separated
+** list of glob patterns, where each glob pattern may optionally be enclosed
+** in "..." or '...' so that it may contain commas.  If a file matches both
+** --include and --exclude then it is excluded.
+**
+** Options:
+**   -X|--exclude GLOBLIST   Comma-separated list of GLOBs of files to exclude
+**   --include GLOBLIST      Comma-separated list of GLOBs of files to include
+**   --name DIRECTORYNAME    The name of the top-level directory in the archive
+**   -R REPOSITORY           Specify a Fossil repository
 */
-void baseline_zip_cmd(void){
+void zip_cmd(void){
   int rid;
   Blob zip;
   const char *zName;
+  Glob *pInclude = 0;
+  Glob *pExclude = 0;
+  const char *zInclude;
+  const char *zExclude;
   zName = find_option("name", 0, 1);
+  zExclude = find_option("exclude", "X", 1);
+  if( zExclude ) pExclude = glob_create(zExclude);
+  zInclude = find_option("include", 0, 1);
+  if( zInclude ) pInclude = glob_create(zInclude);
   db_find_and_open_repository(0, 0);
+
+  /* We should be done with options.. */
+  verify_all_options();
+
   if( g.argc!=4 ){
     usage("VERSION OUTPUTFILE");
   }
-  rid = name_to_typed_rid(g.argv[2],"ci");
+  rid = name_to_typed_rid(g.argv[2], "ci");
+  if( rid==0 ){
+    fossil_fatal("Check-in not found: %s", g.argv[2]);
+    return;
+  }
+
   if( zName==0 ){
     zName = db_text("default-name",
        "SELECT replace(%Q,' ','_') "
@@ -410,44 +491,132 @@ void baseline_zip_cmd(void){
        db_get("project-name", "unnamed"), rid, rid
     );
   }
-  zip_of_baseline(rid, &zip, zName);
+  zip_of_checkin(rid, &zip, zName, pInclude, pExclude);
+  glob_free(pInclude);
+  glob_free(pExclude);
   blob_write_to_file(&zip, g.argv[3]);
+  blob_reset(&zip);
 }
 
 /*
 ** WEBPAGE: zip
-** URL: /zip/RID.zip
+** URL: /zip
 **
-** Generate a ZIP archive for the baseline.
-** Return that ZIP archive as the HTTP reply content.
+** Generate a ZIP archive for the check-in specified by the "r"
+** query parameter.  Return that ZIP archive as the HTTP reply content.
+**
+** Query parameters:
+**
+**   name=NAME[.zip]     The base name of the output file.  The default
+**                       value is a configuration parameter in the project
+**                       settings.  A prefix of the name, omitting the
+**                       extension, is used as the top-most directory name.
+**
+**   r=TAG               The check-in that is turned into a ZIP archive.
+**                       Defaults to "trunk".  This query parameter used to
+**                       be called "uuid" and the older "uuid" name is still
+**                       accepted for backwards compatibility.  If this
+**                       query paramater is omitted, the latest "trunk"
+**                       check-in is used.
+**
+**   in=PATTERN          Only include files that match the comma-separate
+**                       list of GLOB patterns in PATTERN, as with ex=
+**
+**   ex=PATTERN          Omit any file that match PATTERN.  PATTERN is a
+**                       comma-separated list of GLOB patterns, where each
+**                       pattern can optionally be quoted using ".." or '..'.
+**                       Any file matching both ex= and in= is excluded.
 */
 void baseline_zip_page(void){
   int rid;
-  char *zName, *zRid;
+  const char *z;
+  char *zName, *zRid, *zKey;
   int nName, nRid;
-  Blob zip;
+  const char *zInclude;         /* The in= query parameter */
+  const char *zExclude;         /* The ex= query parameter */
+  Blob cacheKey;                /* The key to cache */
+  Glob *pInclude = 0;           /* The compiled in= glob pattern */
+  Glob *pExclude = 0;           /* The compiled ex= glob pattern */
+  Blob zip;                     /* ZIP archive accumulated here */
 
   login_check_credentials();
-  if( !g.perm.Zip ){ login_needed(); return; }
+  if( !g.perm.Zip ){ login_needed(g.anon.Zip); return; }
+  load_control();
   zName = mprintf("%s", PD("name",""));
   nName = strlen(zName);
-  zRid = mprintf("%s", PD("uuid","trunk"));
+  z = P("r");
+  if( z==0 ) z = P("uuid");
+  if( z==0 ) z = "trunk";
+  zRid = fossil_strdup(z);
   nRid = strlen(zRid);
-  for(nName=strlen(zName)-1; nName>5; nName--){
-    if( zName[nName]=='.' ){
-      zName[nName] = 0;
-      break;
+  zInclude = P("in");
+  if( zInclude ) pInclude = glob_create(zInclude);
+  zExclude = P("ex");
+  if( zExclude ) pExclude = glob_create(zExclude);
+  if( nName>4 && fossil_strcmp(&zName[nName-4], ".zip")==0 ){
+    /* Special case:  Remove the ".zip" suffix.  */
+    nName -= 4;
+    zName[nName] = 0;
+  }else{
+    /* If the file suffix is not ".zip" then just remove the
+    ** suffix up to and including the last "." */
+    for(nName=strlen(zName)-1; nName>5; nName--){
+      if( zName[nName]=='.' ){
+        zName[nName] = 0;
+        break;
+      }
     }
   }
-  rid = name_to_typed_rid(nRid?zRid:zName,"ci");
+  rid = name_to_typed_rid(nRid?zRid:zName, "ci");
   if( rid==0 ){
     @ Not found
     return;
   }
   if( nRid==0 && nName>10 ) zName[10] = 0;
-  zip_of_baseline(rid, &zip, zName);
-  free( zName );
-  free( zRid );
+
+  /* Compute a unique key for the cache entry based on query parameters */
+  blob_init(&cacheKey, 0, 0);
+  blob_appendf(&cacheKey, "/zip/%z", rid_to_uuid(rid));
+  blob_appendf(&cacheKey, "/%q", zName);
+  if( zInclude ) blob_appendf(&cacheKey, ",in=%Q", zInclude);
+  if( zExclude ) blob_appendf(&cacheKey, ",ex=%Q", zExclude);
+  zKey = blob_str(&cacheKey);
+
+  if( P("debug")!=0 ){
+    style_header("ZIP Archive Generator Debug Screen");
+    @ zName = "%h(zName)"<br />
+    @ rid = %d(rid)<br />
+    if( zInclude ){
+      @ zInclude = "%h(zInclude)"<br />
+    }
+    if( zExclude ){
+      @ zExclude = "%h(zExclude)"<br />
+    }
+    @ zKey = "%h(zKey)"
+    style_footer();
+    return;
+  }
+  if( referred_from_login() ){
+    style_header("ZIP Archive Download");
+    @ <form action='%R/zip/%h(zName).zip'>
+    cgi_query_parameters_to_hidden();
+    @ <p>ZIP Archive named <b>%h(zName).zip</b> holding the content
+    @ of check-in <b>%h(zRid)</b>:
+    @ <input type="submit" value="Download" />
+    @ </form>
+    style_footer();
+    return;
+  }
+  blob_zero(&zip);
+  if( cache_read(&zip, zKey)==0 ){
+    zip_of_checkin(rid, &zip, zName, pInclude, pExclude);
+    cache_write(&zip, zKey);
+  }
+  glob_free(pInclude);
+  glob_free(pExclude);
+  fossil_free(zName);
+  fossil_free(zRid);
+  blob_reset(&cacheKey);
   cgi_set_content(&zip);
   cgi_set_content_type("application/zip");
 }
