@@ -1,4 +1,4 @@
-#!/bin/sh
+#!/usr/bin/env tclsh
 #
 # Run this TCL script to generate a WIKI page that contains a
 # permuted index of the various documentation files.
@@ -8,6 +8,7 @@
 
 set doclist {
   aboutcgi.wiki {How CGI Works In Fossil}
+  aboutdownload.wiki {How The Download Page Works}
   adding_code.wiki {Adding New Features To Fossil}
   adding_code.wiki {Hacking Fossil}
   antibot.wiki {Defense against Spiders and Bots}
@@ -33,11 +34,12 @@ set doclist {
   event.wiki {Events}
   faq.wiki {Frequently Asked Questions}
   fileformat.wiki {Fossil File Format}
-  fiveminutes.wiki {Update and Running in 5 Minutes as a Single User}
+  fiveminutes.wiki {Up and Running in 5 Minutes as a Single User}
   foss-cklist.wiki {Checklist For Successful Open-Source Projects}
   fossil-from-msvc.wiki {Integrating Fossil in the Microsoft Express 2010 IDE}
   fossil-v-git.wiki {Fossil Versus Git}
   hacker-howto.wiki {Hacker How-To}
+  hashpolicy.wiki {Hash Policy: Choosing Between SHA1 and SHA3-256}
   /help {Lists of Commands and Webpages}
   hints.wiki {Fossil Tips And Usage Hints}
   index.wiki {Home Page}
@@ -46,7 +48,7 @@ set doclist {
   /md_rules {Markdown Formatting Rules}
   newrepo.wiki {How To Create A New Fossil Repository}
   password.wiki {Password Management And Authentication}
-  pop.wiki {Principles Of Operations}
+  pop.wiki {Principles Of Operation}
   private.wiki {Creating, Syncing, and Deleting Private Branches}
   qandc.wiki {Questions And Criticisms}
   quickstart.wiki {Fossil Quick Start Guide}
@@ -80,17 +82,20 @@ set doclist {
 }
 
 set permindex {}
-set stopwords {fossil and a in of on the to are about used by for or}
+set stopwords {
+   a about against and are as by for fossil from in of on or should the to use
+   used with
+}
 foreach {file title} $doclist {
   set n [llength $title]
   regsub -all {\s+} $title { } title
-  lappend permindex [list $title $file]
+  lappend permindex [list $title $file 1]
   for {set i 0} {$i<$n-1} {incr i} {
     set prefix [lrange $title 0 $i]
     set suffix [lrange $title [expr {$i+1}] end]
     set firstword [string tolower [lindex $suffix 0]]
     if {[lsearch $stopwords $firstword]<0} {
-      lappend permindex [list "$suffix &mdash; $prefix" $file]
+      lappend permindex [list "$suffix &mdash; $prefix" $file 0]
     }
   }
 }
@@ -120,7 +125,8 @@ book</a>
 <h2>Permuted Index:</h2>
 <ul>}
 foreach entry $permindex {
-  foreach {title file} $entry break
+  foreach {title file bold} $entry break
+  if {$bold} {set title <b>$title</b>}
   if {[string match /* $file]} {set file ../../..$file}
   puts $out "<li><a href=\"$file\">$title</a></li>"
 }
